<?php

namespace BulkUpload\Handlers;

use BulkUpload\Error\Error;

/**
 * Handles all the product metadata, with crud methods.
 */
class MetaData
{

    /**
     * Retrives product metadata from the provided $product_id if exists, if not then returns an Error.
     *
     * @param integer $product_id
     * @param string $meta_key
     * @return array|Error
     */
    static public function getMetaData(int $product_id, string $meta_key):array|Error{

        $db = new \collection;
        $where = [
            'post_id' => $product_id,
            'meta_key' => $meta_key
        ];
        $result = $db->search(str_replace('OvMdW_', TABLE_PREFIX,'OvMdW_postmeta'), $where, limit:1);
        if($result['cant_rows'] < 1){
            return new Error('Meta Data', 'There is no meta data that matches the given product id and meta key.');
        }
        return $result[0];
    }

    /**
     * Inserts a new meta int the OvMdW_postmeta table for the provided $product_id withthe given information.
     *
     * @param integer $product_id
     * @param string $meta_key
     * @param string $meta_value
     * @return void
     */
    static public function insertMetaData(int $product_id, string $meta_key, string $meta_value):void{

        $db = new \collection;
        $insert_data = [
            'post_id' => $product_id,
            'meta_key' => $meta_key,
            'meta_value' => $meta_value
        ];

        $db->insert(str_replace('OvMdW_', TABLE_PREFIX,'OvMdW_postmeta'), $insert_data);
        return;

    }

    /**
     * Updates the $meta_value for the product with the provided $product_id and the matched $meta_key.
     *
     * @param integer $product_id
     * @param string $meta_key
     * @param string $meta_value
     * @return void
     */
    static public function updateMetaData(int $product_id, string $meta_key, string $meta_value):void{

        $db = new \collection;
        $where = [
            'post_id' => $product_id,
            'meta_key' => $meta_key,
        ];
        $updateData = [
            'meta_value' => $meta_value
        ];

        $db->update(str_replace('OvMdW_', TABLE_PREFIX,'OvMdW_postmeta'), $where, $updateData);
        return;

    }

    /**
     * Deletes the metadata based on $product_id and $meta_key or $meta_id.
     *
     * @param integer $product_id
     * @param string|null $meta_key
     * @param integer|null $meta_id
     * @return void
     */
    static public function deleteMetaData(?int $product_id, ?string $meta_key, ?int $meta_id):void{

        $db = new \collection;

        if($meta_id){
            $where = [
                'post_id' => $product_id,
                'meta_key' => $meta_key,
            ];
        } else {
            $where = [
                'meta_id' => $meta_id
            ];
        }

        $db->delete(str_replace('OvMdW_', TABLE_PREFIX,'OvMdW_postmeta'), $where);
        return;

    }

}