<?php

namespace BulkUpload;

use BulkUpload\Error\Error;

class Tariff 
{

    public ?int $id;
    public string $code;
    public string $name;
    public bool $status;
    public int $dai;
    public int $itbms;
    public int $isc;
    public int $iccdp;

    public function __construct(array $tariff)
    {
        
        $this->id = $tariff['id_hs_code'];
        $this->code = $tariff['dmce_hs_codes'];
        $this->name = $tariff['hs_code_description'];
        $this->status = $tariff['status'];
        $this->dai = $tariff['dai'];
        $this->itbms = $tariff['itbms'];
        $this->isc = $tariff['isc'];
        $this->iccdp = $tariff['iccdp'];

    }

    /**
     * Retrives the tariff name/descrpition of a certaing given code. Returns name if exists and Error if the code is not found.
     *
     * @param \collection $db
     * @param string|integer $code
     * @return string|Error
     */
    static public function getTariffNameFromCode(string|int $code):string|Error{

        $db = new \collection;
        $response = $db->search('ovmdw_dmce_hs_codes_complete', ['dmce_hs_codes' => $code, 'status' => 1], limit:'1');

        if($response['cant_rows'] < 1){

            return new Error("Tariff", "Tariff name provided doesn't exist in the database or is disabled.");

        }


        return  $response[0]['hs_code_description']??new Error("Tariff", "Tariff code is empty in the database.");

    }

    /**
     * Retrieves a Tariff instance, base on the given code. If code is not found, then retrieves an Error object.
     *
     * @param \collection $db
     * @param string|integer $code
     * @return Tariff|Error
     */
    static public function getTariffFromCode(string $code):Tariff|Error{

        $db = new \collection;

        $response = $db->search('ovmdw_dmce_hs_codes_complete', ['dmce_hs_codes' => $code, 'status' => 1], limit:'1');
        if($response['cant_rows'] < 1){

            return new Error("Tariff", "Tariff code provided doesn't exist in the database or is disabled.");

        }

        //die(print_r($response[0]));
        return new Tariff($response[0]);

    }


}