<?php

namespace BulkUpload\Terms;

use BulkUpload\Error\Error;
use BulkUpload\Interfaces\TermInterface;

class Tag /*implements TermInterface*/
{

    private int $term_id;
    private int $term_taxonomy_id;
    private string $name;
    private string $slug;
    private string $taxonomy = "product_tag";
    private ?int $count;

    /**
     * Instance of the database manager
     *
     * @var \collection
     */
    private \collection $db;

    /**
     * Instances a new Tag class with a collection propertie to manage the database.
     */
    public function __construct(array $tag_data)
    {
        $this->term_id = $tag_data['term_id'];
        $this->term_taxonomy_id = $tag_data['term_taxonomy_id'];
        $this->name = $tag_data['name'];
        $this->slug = $tag_data['slug'];
        $this->count = $tag_data['count'];
        $this->db = new \collection;
    }

    /**
     * Returns the term id.
     *
     * @return integer
     */
    public function getTermTermId()
    {
        return $this->term_id;
    }

    /**
     * Returns the term name.
     *
     * @return string
     */
    public function getTermName()
    {
        return $this->name;
    }

    /**
     * Returns a Tag|Tag class that matches with the provided id.
     *
     * @param integer $term_id
     * @return Tag|Error
     */
    static public function getTermFromId(int $term_id): mixed
    {

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_tag'
                    AND OvMdW_terms.`term_id` = '" . $db->sanitizeData($term_id) . "';
        ");

        $result = $db->plainQuery($query);
        if ($result['cant_rows'] < 1) {
            return new Error("Tag", "There is no tag with the provided term_id");
        }

        return new Tag($result[0]);
    }

    /**
     * Returns a Tag|Tag class that matches with the provided name and type.
     *
     * @param string $name
     * @param string $type
     * @return Tag|Error
     */
    static public function getTermFromName(string $name): mixed
    {

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count,
                        OvMdW_term_taxonomy.parent AS parent
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_tag'
                    AND OvMdW_terms.`name` = '" . $db->sanitizeData($name) . "';
        ");

        $result = $db->plainQuery($query);
        if ($result['cant_rows'] < 1) {
            return new Error("Tag", "There is no tag with the provided name");
        }

        return new Tag($result[0]);
    }

    static public function getTermFromSlug(string $slug): Tag|Error
    {

        $db = new \collection;
        $query = str_replace('OvMdW_', TABLE_PREFIX, "  SELECT
                        OvMdW_terms.term_id AS term_id,
                        OvMdW_terms.`name` AS `name`,
                        OvMdW_terms.slug AS slug,
                        OvMdW_term_taxonomy.term_taxonomy_id AS term_taxonomy_id,
                        OvMdW_term_taxonomy.taxonomy AS taxonomy,
                        OvMdW_term_taxonomy.count AS count
                    FROM
                        OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
                    WHERE
                        OvMdW_term_taxonomy.taxonomy = 'product_tag'
                    AND OvMdW_terms.`slug` = '" . $db->sanitizeData($slug) . "';
        ");

        $result = $db->plainQuery($query);
        if ($result['cant_rows'] < 1) {
            return new Error("Tag", "There is no tag with the provided slug");
        }

        return new Tag($result[0]);
    }
}
