<?php

class Cart
{

    private int|null $cart_id = null;
    private string $user_id = '';
    private \collection $db;
    public array $cart;
    private string|null $hash = '';
    private array $unavailable_products = [];

    public function __construct($user_id, $hash)
    {
        $this->db = new collection();
        $this->cart = [
            "cart_items" => [],
            "date" => "",
            "hash" => "",
            "id_cart" => 0,
            "id_user" => 0,
            "status" => 1
        ];
        if (isset($user_id) && $user_id != '') {
            $this->user_id = $user_id;
            if (isset($hash) && $hash != '' && $hash != 'null') {
                $this->hash = $hash;
            }
            $this->createCart();
            $this->getCartId();
            return;
        }

        if (isset($hash) && $hash != '' && $hash != 'null') {
            $this->user_id = $hash;
            $this->hash = $hash;
            $this->createCart();
            $this->getCartId();
            return;
        }
    }

    private function getCartId(): void
    {

        if ($this->user_id != $this->hash && $this->hash != "") {
            $old_cart = $this->db->searchForce('ng_cart', ['id_user' => $this->hash, 'status' => 1]);
            if ($old_cart['cant_rows'] > 0) {
                $id_old_cart = $old_cart[0]['id_cart'];
                $old_items_cart = $this->db->searchForce('ng_cart_items', ['cart_id' => $id_old_cart]);
                for ($i = 0; $i < $old_items_cart['cant_rows']; $i++) {
                    $old_items_cart[$i]['cart_id'] = $this->cart_id;
                    $this->insert($old_items_cart[$i]);
                }
                $this->destroy($this->hash);
            }
        }

        return;
    }

    private function generateHash(string|null $hash): string
    {
        if (!isset($hash) || $hash == '' || $hash == 'null') {
            $user_id = wp_generate_uuid4();
            $hash = $user_id;
        } else {
            $user_id = $hash;
            $this->user_id = $hash;
        }

        return $user_id;
    }

    /**
     * Gets the basic data stored in the `ng_cart` table
     *
     * @return array
     */
    private function getCartInfo(): array
    {

        $query = "SELECT * FROM `ng_cart` WHERE `id_cart`= " . $this->db->sanitizeData($this->cart_id) . "";
        $cart_info = $this->db->plainQuery($query, force: true);

        if ($cart_info['cant_rows'] < 1) {
            return [];
        }

        return $cart_info[0];
    }

    /**
     * Return an array of the cart items of the provided cart_id.
     *
     * @param integer $cart_id
     * @return array $cart_items
     */
    private function getDatabaseCartItems(): void
    {

        $where = ['cart_id' => $this->cart_id];
        $cart_items = $this->db->searchForce('ng_cart_items', $where);

        $final_items['hash'] = $this->hash;
        foreach ($this->getCartInfo() as $key => $val) {
            $final_items[$key] = $val;
        }

        if ($cart_items < 1) {
            return;
        }
        $final_items['cart_items'] = [];
        foreach ($cart_items as $key => $cart_item) {
            if ($key == 'cant_rows') {
                continue;
            }

            if (is_array($cart_item)) {
                $final_items['cart_items'][] = $cart_item;
            }
        }

        $this->cart = $final_items;
    }

    /**
     * Looks for the cart of the given userm if there is no cart, then return falsem if exists, returns true.
     *
     * @return bool
     */
    private function cart_exists(): bool
    {
        if ($this->user_id == "") {
            $this->user_id = $this->generateHash(null);
            $this->hash = $this->user_id;
        }
        $cart_id = $this->db->searchForce('ng_cart', ['id_user' => $this->user_id, 'status' => 1], limit: 1);
        if ($cart_id['cant_rows'] > 0) {
            $this->cart_id = $cart_id[0]['id_cart'];
            return true;
        }
        return false;
    }

    /**
     * Checks if the cart exists, if not then creates a new cart and assigns it to the user.
     *
     * @return void
     */
    private function createCart(): void
    {
        //die(var_dump($this));
        if (!$this->cart_exists()) {
            //die(var_dump($this));
            $this->cart_id = $this->db->insert('ng_cart', ['id_user' => $this->user_id, 'date' => date('Y-m-d H:i:s'), 'status' => 1]);
            return;
        }
        return;
    }

    /**
     * Takes the products in the cart of the database and return the info of those products.
     *
     * @return array
     */
    public function getProductInfo(): array
    {
        $products = new Products;
        $info=[];
        $list = $products->searchById(array_map(function ($item) {
            return $item['product_id'];
        }, $this->cart['cart_items']), true, true);
        foreach ($list as $value) {
            $info[$value['product_id']]=$value;
        }
        return $info;
    }

    /**
     * Takes an array of the product_id, cart_id, quantity and variation id and inserts the data on the cart id give.
     *
     * @param array $cart_items
     * @return void
     */
    public function insert(array $cart_item): void
    {
        if (!$this->cart_id) {
            $this->createCart();
        }
        $item = $this->db->searchForce('ng_cart_items', ['cart_id' => $this->cart_id, 'product_id' => $cart_item['product_id'], 'variation_id' => $cart_item['variation_id']], limit: 1);
        if ($item['cant_rows'] > 0) {
            $quantity = $cart_item['quantity'] + $item[0]['quantity'];
            $this->update($cart_item['product_id'], $quantity, $cart_item['variation_id']);
        } else {
            $cart_item['cart_id'] = $this->cart_id;
            $this->db->insert('ng_cart_items', $cart_item);
        }
        return;
    }

    /**
     * Takes the id of the cart item and modifies the quantity of that item.
     *
     * @param int|string  $product_id
     * @param int $value
     * @param int|string $variation_id
     * @return void
     */
    public function update(int|string  $product_id, int $value, int|string $variation_id): void
    {

        $where = ['cart_id' => $this->cart_id, 'product_id' => $product_id, 'variation_id' => $variation_id];
        $values = ['quantity' => $value];

        $this->db->update('ng_cart_items', $where, $values);
    }

    public function getItems(bool $getStatus = false): array
    {

        if (!$this->cart_id) {
            return $this->cart;
        }
        //Asigna los productos de la base de datos al carrito temporal
        $this->getDatabaseCartItems();

        //Si no hay productos no seguir
        if (count($this->cart['cart_items']) < 1) {
            return $this->cart;
        }

        $list = [];
        //Obtener info de todos los productos temporales

        $all_products_info = $this->getProductInfo();

        foreach($this->cart['cart_items'] as $item){

            //Si no hay informacion para este producto, deshabilitalo
            if(!isset($all_products_info[$item['product_id']])){

                if ($getStatus && ($item['status'] == 1 || $item['status'] == "1")) {
                    $this->updateItemStatus($item['product_id'], $item['variation_id'], "0");
                    $this->unavailable_products[] = $item['product_id'];
                }
                continue;
            }

            //En caso de que haya informacion, ejectuta las siguientes comprobaciones:

                
            //Revisa si este item es un producto simple
            if(intval($item['variation_id']) == 0){

                //Asigna y guarda el producto
                $item['info'] = $all_products_info[$item['product_id']];
                $list[] = $item;

                //De ser necesario actualiza el estado
                if ($getStatus && ($item['status'] == 0 || $item['status'] == "0")) {
                    $this->updateItemStatus($item['product_id'], $item['variation_id'], "1");
                }

                continue;
            } 

            //Si se llego hasta aca es porque el item es hijo de un producto variable

            //Revisa si el variation id existe dentro del producto padre
            if(!$this->isExistingVariation($item['variation_id'], $all_products_info[$item['product_id']]['variations'])){

                //Desactivar el producto porque no hay informacion para dicha variacion  
                if ($getStatus && ($item['status'] == 1 || $item['status'] == "1")) {
                    $this->updateItemStatus($item['product_id'], $item['variation_id'], "0");
                    $this->unavailable_products[] = $item['product_id'];
                }
                
                continue;
            }

            //Asigna y guarda el producto
            $item['info'] = $all_products_info[$item['product_id']];
            $list[] = $item;

            //De ser necesario actualiza el estado
            if ($getStatus && ($item['status'] == 0 || $item['status'] == "0")) {
                $this->updateItemStatus($item['product_id'], $item['variation_id'], "1");
            }

            
        }

        $this->cart['cart_items'] = $list;
        return $this->cart;
    }

    private function isExistingVariation($id, $array):bool 
    {
        $status = false;
        foreach ($array as $val) {
            if ($val['product_id'] == $id) {
                $status = true;
            }
        }
        return $status;
     }

    /**
     * Gets the item row id and deletes it from the database
     *
     * @param int $row_id
     * @return void
     */
    public function remove($product_id, $variation_id): void
    {
        $where = ['cart_id' => $this->cart_id, 'product_id' => $product_id, 'variation_id' => $variation_id];
        $this->db->delete('ng_cart_items', $where);
        return;
    }

    /**
     * Sets a logic remove of the carts, changing the status.
     *
     * @return void
     */
    public function destroy($cart_id = ''): void
    {
        if (isset($cart_id) && $cart_id != '') {
            $where = [
                'id_user' => $cart_id
            ];
        } else {
            $where = [
                'id_cart' => $this->cart_id
            ];
        }
        $values = ['status' => 0];
        $this->db->update('ng_cart', $where, $values);
        $this->createCart();
        return;
    }

    /**
     * Check if all the products in the cart exist, are publish or have visibility, if not,
     * then remove them from the database.
     *
     * @return void
     */
    private function solveConflicts(): void
    {
        //die(var_dump($this->cart['cart_items']));
        foreach ($this->cart['cart_items'] as $key => $item) {
            if (!isset($item['info']) || $item['info'] == "" || $item['info'] == []) {
                //$this->remove($this->cart['cart_items'][$key]['product_id'], $this->cart['cart_items'][$key]['variation_id']);
                if ($item['status'] == 1 || $item['status'] == "1") {
                    $this->updateItemStatus($item['product_id'], $item['variation_id'] ?? 0, "0");
                    $this->unavailable_products[] = $item['product_id'];
                }
                unset($this->cart['cart_items'][$key]);
            } else {
                if ($item['status'] == 0 || $item['status'] == "0") {
                    $this->updateItemStatus($item['product_id'], $item['variation_id'] ?? 0, "1");
                }
            }
        }
    }

    private function updateItemStatus(int|string $product_id, int|string $variation_id, string $status): void
    {

        $where = ['cart_id' => $this->cart_id, 'product_id' => $product_id, 'variation_id' => $variation_id];
        $values = ['status' => $status];

        $this->db->update('ng_cart_items', $where, $values);
    }

    public function getCartItemsStatus(): array
    {
        $cart = $this->getItems(true);
        return $this->unavailable_products;
    }
}
