<?php

namespace InventoryManagement;

use DateTime;

class ProductManager
{

    use InventoryStatusManager;
    use InventoryCronBM;
    use InventoryCronVendor;

    private \collection $db;

    public function __construct()
    {
        $this->db = new \collection;
    }

    /**
     * Sets the list of products assigned to the given vendor.
     *
     * @param array $vendorData
     * @return void
     */
    public function getVendorProducts(string $identifier, int $page, int $limit): array
    {

        $pd = new \Products();

        $result = $this->db->searchForce('ng_inventory_tracker', ['still_working' => '1', 'identifier' => $identifier]);

        if ($result['cant_rows'] < 1) {
            throw new \Exception("No hay ningun id de usuario coincidente en su solicitud", 1);
        }

        $date = new DateTime($result[0]['last_call']);
        $products = $pd->searchByVendor($result[0]['id_user'], $page, $limit, "AND OvMdW_posts.post_date <= '" . $date->format("Y-m-d H:i:s") . "'");

        return $products;
    }

    private function saveInventoryLog(string $identifier, array $activeProducts, array $inactiveProducts)
    {

        $currentDate = new DateTime("now");
        $data = [
            'action_token' => $identifier,
            'publish_products' => implode(",", $activeProducts),
            'draft_products' => implode(",", $inactiveProducts),
            'date' => $currentDate->format("Y-m-d")
        ];

        $this->db->insert('ng_inventory_tracker_log', $data);
    }

    public function handleProductState(string $identifier, array $inactiveProducts = [], string $action = 'draft', bool $fast = false): void
    {

        if ($identifier == "") {
            throw new \Exception("No ha otorgado un identificador valido");
        }

        $id_user = $this->getUserFromIdentifier($identifier, $this->db);
        $cronDate = $this->getCronDateFromIdentifier($identifier, $this->db);

        if (!empty($inactiveProducts)) {
            $this->db->plainQuery("UPDATE  OvMdW_posts
                                SET OvMdW_posts.post_status = '" . $action . "'
                                WHERE
                                OvMdW_posts.post_type = 'product' AND
                                OvMdW_posts.post_status = 'publish' AND
                                OvMdW_posts.post_date <= '" . $cronDate->format("Y-m-d H:i:s") . "' AND
                                OvMdW_posts.post_author = " . $id_user . " AND
                                OvMdW_posts.ID IN (" . implode(",", $inactiveProducts) . ")");
        }
        if($fast){
            $this->db->plainQuery("UPDATE  OvMdW_posts
            SET OvMdW_posts.post_status = '" . $action . "'
            WHERE
            OvMdW_posts.post_type = 'product' AND
            OvMdW_posts.post_status = 'publish' AND
            OvMdW_posts.post_date <= '" . $cronDate->format("Y-m-d H:i:s") . "' AND
            OvMdW_posts.post_author = " . $id_user );
        }
        $this->completeInventoryManagement($identifier, $this->db);
    }
}
