<?php 

class Post {

    private array $data;
    private collection $db;
    private $offset;
    private $limit;
    private $queneu_list = [];

    public function __construct($offset = 0, $limit = 0)
    {
        $this->db = new collection();
        $this->offset = $offset;
        $this->limit = $limit;

    }

    public function getPosts(array $skip = []){

        if(isset($skip) && count($skip) > 0){
            $data = $this->queryPosts($this->offset, $this->limit, $skip);
        } else {
            $data = $this->queryPosts($this->offset, $this->limit);
        }

        if(count($data) < 1){
            throw new Exception("No hay posts disponibles para mostrar.", 1); 
        }
        return $data;

    }

    public function getPost(string $post_name){

        $data = $this->queryPost($post_name);

        if(count($data) < 1){
            throw new Exception("No hay post disponible para mostrar.", 1); 
        }
        return $data;

    }

    /**
     * Make a query for the BM DB that returns all the posts published.
     *
     * @return array
     */
    private function queryPosts(int $offset, int $limit, array $skip = []){

        if(!is_int($offset) || $offset < 0){
            throw new Exception("The offset is not a valid number", 1);     
        }

        if(!is_int($limit) || $limit < 0) {
            throw new Exception("The limit is not a valid number", 1);
        }

        if(isset($skip) && count($skip) > 0){
            foreach ($skip as $key => $value) {
                $skip[$key]= "'".$this->db->sanitizeData($value)."'";
            }
            $list = implode(",", $skip);
        } else {
            $list = '""';
        }
        
        $query = 'SELECT
        OvMdW_posts.ID,
        OvMdW_posts.post_author,
        OvMdW_posts.post_date,
        OvMdW_posts.post_content,
        OvMdW_posts.post_title,
        OvMdW_posts.post_excerpt,
        OvMdW_posts.post_status,
        OvMdW_posts.post_name,
        OvMdW_posts.post_modified,
        OvMdW_posts.post_parent,
        OvMdW_posts.post_type
        FROM
        OvMdW_posts
        INNER JOIN OvMdW_term_relationships ON OvMdW_term_relationships.object_id = OvMdW_posts.ID
        INNER JOIN OvMdW_term_taxonomy ON OvMdW_term_taxonomy.term_taxonomy_id = OvMdW_term_relationships.term_taxonomy_id
        INNER JOIN OvMdW_terms ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
        WHERE
        OvMdW_posts.post_status = "publish" AND
        OvMdW_posts.post_type = "post" AND
        OvMdW_posts.post_name NOT IN ('.$list.') AND
        OvMdW_terms.slug != "how-to"
        GROUP BY OvMdW_posts.ID
        ORDER BY
        OvMdW_posts.post_date DESC
        LIMIT
        '.$offset.', '.$limit.'
        ';

        //die($query);

        $prepared_query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            $query
        );
        $data = $this->db->plainQuery($prepared_query, false, true);
        $finalData = [];
        foreach($data as $key => $val){
            if($key == 'cant_rows'){
                continue;
            }

            $val['descp'] = $this->wp_strip_all_tags($val['post_content']); 
            $val['post_title'] = strip_tags($val['post_title']);
            $val['image'] = $this->getMainImage($val['ID']);
            array_push($finalData, $val);
        }
        return $finalData;

    }

    public function wp_strip_all_tags( $text, $remove_breaks = false ) {
        if ( is_null( $text ) ) {
            return '';
        }
    
        if ( ! is_scalar( $text ) ) {
            /*
             * To maintain consistency with pre-PHP 8 error levels,
             * trigger_error() is used to trigger an E_USER_WARNING,
             * rather than _doing_it_wrong(), which triggers an E_USER_NOTICE.
             */
            trigger_error(
                sprintf(
                    /* translators: 1: The function name, 2: The argument number, 3: The argument name, 4: The expected type, 5: The provided type. */
                    'Warning: %1$s expects parameter %2$s (%3$s) to be a %4$s, %5$s given.',
                    __FUNCTION__,
                    '#1',
                    '$text',
                    'string',
                    gettype( $text )
                ),
                E_USER_WARNING
            );
    
            return '';
        }
    
        $text = preg_replace( '@<(script|style)[^>]*?>.*?</\\1>@si', '', $text );
        $text = strip_tags( $text );
    
        if ( $remove_breaks ) {
            $text = preg_replace( '/[\r\n\t ]+/', ' ', $text );
        }
    
        return trim( $text );
    }

    private function queryPost(string $post_name){

        if(!is_string($post_name) || $post_name == ""){
            throw new Exception("The postname can not be different to string or empty", 1);     
        }
        
        $query = 'SELECT
        OvMdW_posts.ID,
        OvMdW_posts.post_author,
        OvMdW_posts.post_date,
        OvMdW_posts.post_content,
        OvMdW_posts.post_title,
        OvMdW_posts.post_excerpt,
        OvMdW_posts.post_status,
        OvMdW_posts.post_name,
        OvMdW_posts.post_modified,
        OvMdW_posts.post_parent,
        OvMdW_posts.post_type
        FROM
        OvMdW_posts
        INNER JOIN OvMdW_term_relationships ON OvMdW_term_relationships.object_id = OvMdW_posts.ID
        INNER JOIN OvMdW_term_taxonomy ON OvMdW_term_taxonomy.term_taxonomy_id = OvMdW_term_relationships.term_taxonomy_id
        INNER JOIN OvMdW_terms ON OvMdW_terms.term_id = OvMdW_term_taxonomy.term_id
        WHERE
        OvMdW_posts.post_status = "publish" AND
        OvMdW_posts.post_type = "post" AND 
        OvMdW_posts.post_name = "'.$this->db->sanitizeData($post_name).'" AND
        OvMdW_terms.slug != "howto"
        GROUP BY OvMdW_posts.ID
        ORDER BY
        OvMdW_posts.post_date DESC';

        $prepared_query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            $query
        );

        
        //die(var_dump($prepared_query));

        $data = $this->db->plainQuery($prepared_query, false, true);
        //die(var_dump($data));

        $finalData = [];
        foreach($data as $key => $val){
            if($key === 'cant_rows'){
                continue;
            }
            $val['descp'] = $this->wp_strip_all_tags($val['post_content']); 
            $val['image'] = $this->getMainImage($val['ID']);
            $val['post_content'] = $this->processBucketImages($val['post_content']);
            $val['post_title'] = strip_tags($val['post_title']);
            array_push($finalData, $val);
        }

        return $finalData;

    }

    private function getMainImage($id){

        $query = 'SELECT
        ovmdw_postmeta.meta_value
        FROM
        ovmdw_postmeta
        WHERE
        ovmdw_postmeta.post_id IN (SELECT
        OvMdW_posts.ID
        FROM
        OvMdW_posts
        WHERE
        OvMdW_posts.ID = (SELECT  OvMdW_postmeta.meta_value FROM OvMdW_postmeta WHERE OvMdW_postmeta.post_id  = '.$id.' AND OvMdW_postmeta.meta_key = "_thumbnail_id")) AND
        ovmdw_postmeta.meta_key = "_wp_attached_file"
        GROUP BY
        ovmdw_postmeta.meta_id            
        '; 

        $prepared_query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            $query
        );
        $data = $this->db->plainQuery($prepared_query, false, true);

        if($data['cant_rows']<1){
            return "";
        }
        return "https://d1u9bbhcasux7c.cloudfront.net/uploads/".$data[0]['meta_value'];

    }

    private function processBucketImages(string $url):string{

        $new = str_replace("https://bridgemarkets.com/wp-content/uploads/", "https://d1u9bbhcasux7c.cloudfront.net/uploads/", $url);
        return $new;

    }

}