<?php

class Products
{
    private $user_id = 0;
    private $list = [];
    private $id_list = [];
    private $vendors = [];
    private $filter = [];
    private $full_list = [];
    private $attr_list = [];
    private $queneu_list = [];
    private $meta_fields = [
        '_sku' => 'sku',
        '_weight' => 'weight',
        '_length' => 'length',
        '_width' => 'width',
        '_height' => 'height',
        '_stock_status' => 'stock_status',
        '_wc_average_rating' => 'average_rating',
        '_wc_review_count' => 'rating_count',
        '_dokan_wholesale_meta' => 'price,moq',
        '_custom_product_arancel' => 'product_arancel',
        '_custom_product_brand' => 'product_brand',
        '_custom_product_model' => 'product_model',
        'product_origin_country' => 'product_origin_country',
        '_visibility' => 'catalog_visibility',
        '_package_weight' => 'package_weight',
        '_package_length' => 'package_length',
        '_package_width' => 'package_width',
        '_package_height' => 'package_height',
        '_stock' => 'stock',
        '_product_attributes' => 'attributes',
        '_default_attributes' => 'default_attributes',
        '_custom_attr_id' => 'custom_attr_id',
        'tier_options' => 'tier_options',
        'isMultiArancel' => 'isMultiArancel',
        'bulkNumber' => 'bulkNumber',
    ];
    private $minimun_meta_fields = [
        '_sku' => 'sku',
        '_dokan_wholesale_meta' => 'price,moq',
        '_custom_product_brand' => 'product_brand',
        '_wc_average_rating' => 'average_rating',
        '_wc_review_count' => 'rating_count',
    ];
    private $db;

    public function __construct()
    {
        $this->db = new collection();
        try {
            $auth = new auth();
            if ($auth->verify($auth->found())->status()) {
                $this->user_id = $auth->getUser();
            }
        } catch (\Throwable $th) {
            $this->user_id = 0;
        }
    }

    public function getProduct($keyword = '')
    {
        $this->findProduct($keyword);
        $this->getVendorData();
        $this->getChildren();
        $this->getQuickMeta();
        $this->getMeta();
        $this->getChildrenAttr();
        $this->getOrderedCategories();
        $this->getPrimaryImages();
        $this->loadAttributes();
        $this->compact();
        $this->setDefaultImage();
        return $this->list;
    }

    public function searchById(array $ids, $quick = false, bool $force = false)
    {
        $this->findProductByIds($ids, force:$force);
        if ($quick) {
            $this->getVendorData();
            $this->getChildren();
        }
        $this->getQuickMeta();
        $this->getMeta();
        if ($quick) {
            $this->getChildrenAttr();
            $this->getCategories();
            $this->loadAttributes();
        }
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        return $this->list;
    }

    public function search($keyword = '', $page = 1, $count = 12)
    {
        $this->search_routine($keyword, $page, $count);
        //$this->getChildren();
        $this->getQuickMeta();
        $this->getMeta(true);
        $this->getCategories(true);
        $this->getVendorData();
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        $this->FilterData();
        $this->generateFilterData();
        if((($page - 1) * $count) > count($this->list)){
            $page=1;
        }
        return array('products' => array_slice($this->list, ($page - 1) * $count, $count), 'count' => count($this->list), 'filter' => $this->filter);
    }

    private function FilterData()
    {
        $cats = new Categories();
        $firstLevelCategories=$cats->getAllData(false)->getCategories();
        $this->filter['selected']['price_min'] = (isset($_GET['price_min'])) ? $this->db->sanitizeData($_GET['price_min']) : '';
        $this->filter['selected']['price_max'] = (isset($_GET['price_max'])) ? $this->db->sanitizeData($_GET['price_max']) : '';
        $this->filter['selected']['category_filter'] = (isset($_GET['category_filter'])) ? $this->db->sanitizeData($_GET['category_filter']) : '';
        $this->filter['selected']['sub_category_filter'] = (isset($_GET['sub_category_filter'])) ? $this->db->sanitizeData($_GET['sub_category_filter']) : '';
        $this->filter['selected']['vendor_filter'] = (isset($_GET['vendor_filter'])) ? $this->db->sanitizeData($_GET['vendor_filter']) : '';
        $this->filter['selected']['order_by'] = (isset($_GET['order_by'])) ? $this->db->sanitizeData($_GET['order_by']) : '';
        $this->filter['selected']['product_brand'] = (isset($_GET['product_brand'])) ? $this->db->sanitizeData($_GET['product_brand']) : '';

        foreach ($this->list as $key => $product) {
            if ($this->filter['selected']['price_max'] != '' && $product['price'] > $this->filter['selected']['price_max']) {
                unset($this->list[$key]);
                continue;
            }
            if ($this->filter['selected']['price_min'] != '' && $product['price'] < $this->filter['selected']['price_min']) {
                unset($this->list[$key]);
                continue;
            }
            if ($this->filter['selected']['category_filter'] != '' &&  $this->db->sanitizeData($product['categories'][0]['name']) != $this->filter['selected']['category_filter']) {
                unset($this->list[$key]);
                continue;
            }
            $categories  = array_map(function ($p) {
                return $this->db->sanitizeData($p['name']);
            }
            ,$product['categories']);
            if ($this->filter['selected']['sub_category_filter'] != '' &&  (!in_array($this->filter['selected']['sub_category_filter'],$categories) || $product['categories'][1]['name']=='')) {
                unset($this->list[$key]);
                continue;
            }
            if ($this->filter['selected']['vendor_filter'] != ''  && ($this->db->sanitizeData($product['vendor']) != $this->filter['selected']['vendor_filter'] || $product['vendor']=='')) {
                unset($this->list[$key]);
                continue;
            }
            if ($this->filter['selected']['product_brand'] != ''  && ($this->db->sanitizeData($product['product_brand']) != $this->filter['selected']['product_brand'] || $product['product_brand']=='')) {
                unset($this->list[$key]);
                continue;
            }
        }
        $this->filter['selected']['price_min'] = $this->db->unsanitizeData($this->filter['selected']['price_min']);
        $this->filter['selected']['price_max'] = $this->db->unsanitizeData($this->filter['selected']['price_max']);
        $this->filter['selected']['category_filter'] = $this->db->unsanitizeData($this->filter['selected']['category_filter']);
        $this->filter['selected']['sub_category_filter'] = $this->db->unsanitizeData($this->filter['selected']['sub_category_filter']);
        $this->filter['selected']['vendor_filter'] = $this->db->unsanitizeData($this->filter['selected']['vendor_filter']);
        $this->filter['selected']['order_by'] = $this->db->unsanitizeData($this->filter['selected']['order_by']);
        $this->filter['selected']['product_brand'] = $this->db->unsanitizeData($this->filter['selected']['product_brand']);

        switch ($this->filter['selected']['order_by']) {
            case 'Precio Menor a mayor':
                usort($this->list, fn ($a, $b) => $a['price'] >= $b['price']);
                break;

            case 'Precio Mayor a menor':
                usort($this->list, fn ($a, $b) => $a['price'] <= $b['price']);
                break;
            case 'A-Z':
                usort($this->list, fn ($a, $b) => $a['name'] >= $b['name']);
                break;

            case 'Z-A':
                usort($this->list, fn ($a, $b) => $a['name'] <= $b['name']);
                break;

            default:
                usort($this->list, fn ($a, $b) => $a['name'] >= $b['name']);
                break;
        }
    }

    private function generateFilterData()
    {
        foreach ($this->list as $product) {
            $this->setMinPrice($product['price']);
            $this->setMaxPrice($product['price']);
            $this->setbrand($product['product_brand']);
            $this->appendFilterCategories($product);
            $this->appendFilterVendor($product);
        }
        $this->compactCats();
        $this->compactVendor();
        $this->compactBrands();
    }

    private function compactBrands()
    {
        $cat = $this->filter['product_brand'];
        $this->filter['product_brand'] = [];
        foreach ($cat as $k => $v) {
            $this->filter['product_brand'][] = $v;
        }
    }
    private function compactCats()
    {
        $cat = (isset($this->filter['categories']))?$this->filter['categories']:[];
        $this->filter['categories'] = [];
        $subcat = (isset($this->filter['subcat']))?$this->filter['subcat']:[];
        $this->filter['subcat'] = [];
        foreach ($cat as $k => $v) {
            $this->filter['categories'][] = ["slug" => $k, "name" => $v];
        }
        foreach ($subcat as $k => $v) {
            $this->filter['subcat'][] = ["slug" => $k, "name" => $v];
        }
    }

    private function compactVendor()
    {
        $vendors = $this->filter['vendors'];
        $this->filter['vendors'] = [];
        foreach ($vendors as $k => $v) {
            $this->filter['vendors'][] = ["slug" => $k, "name" => $v];
        }
    }

    private function appendFilterCategories($product)
    {
        $cat = [];
        $subcat = [];
        if (is_array($product['categories'])) {
            if (isset($product['categories'][0])) {
                $this->filter['categories'][$product['categories'][0]['slug']] = $product['categories'][0]['name'];
            }
            if (isset($product['categories'][1])) {
                $this->filter['subcat'][$product['categories'][1]['slug']] = $product['categories'][1]['name'];
            }
        }
    }

    private function appendFilterVendor($product)
    {
        if (isset($product['vendor'])) {
            $this->filter['vendors'][$product['vendor_slug']] = $product['vendor'];
        }
    }

    private function setbrand($brand)
    {
        if (!isset($this->filter['product_brand'])) {
            if (!isset($this->filter['product_brand'][$brand]) && $brand != '') {
                $this->filter['product_brand'][$brand] = $brand;
            }
        }
        if (!isset($this->filter['product_brand'][$brand]) && $brand != '') {
            $this->filter['product_brand'][$brand] = $brand;
        }
    }
    private function setMinPrice($price = 0)
    {
        if (!isset($this->filter['price']['min'])) {
            $this->filter['price']['min'] = $price;
        }
        if ($this->filter['price']['min'] > $price) {
            $this->filter['price']['min'] = $price;
        }
    }
    private function setMaxPrice($price = 0)
    {
        if (!isset($this->filter['price']['max'])) {
            $this->filter['price']['max'] = $price;
        }
        if ($this->filter['price']['max'] < $price) {
            $this->filter['price']['max'] = $price;
        }
    }


    public function latest($number = 60, $page = 1, $count = 12)
    {
        $this->search_routine_last($number, $page, $count);
        $this->getQuickMeta();
        $this->getMeta(true);
        $this->getCategories(true);
        $this->getVendorData();
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        $this->FilterData();
        $this->generateFilterData();
        return array('products' => array_slice($this->list, ($page - 1) * $count, $count), 'count' => count($this->list), 'filter' => $this->filter);
    }

    public function searchByVendor($vendor_id = '', $page = 1, $count = 12)
    {
        $this->search_routine_by_vendor($vendor_id, $page, $count);
        $this->getChildren();
        $this->getQuickMeta();
        $this->getMeta();
        $this->getCategories(true);
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        return array('products' => $this->list, 'count' => count($this->id_list));
    }

    public function searchByCat($keyword = '', $page = 1, $count = 12)
    {
        $this->search_category_routine($keyword, $page, $count);
        $this->getQuickMeta();
        $this->getMeta(true);
        $this->getCategories(true);
        $this->getVendorData();
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        $this->FilterData();
        $this->generateFilterData();
        return array('products' => array_slice($this->list, ($page - 1) * $count, $count), 'count' => count($this->list), 'filter' => $this->filter);
    }


    public function getRandomProducts($count = 2)
    {
        $this->search_random_routine($count);
        $this->getQuickMeta();
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        return array('products' => $this->list, 'count' => count($this->id_list));
    }

    private function setDefaultImage()
    {

        for ($i = 0; $i < count($this->list); $i++) {
            if (count($this->list[$i]['images']) == 0) {
                $this->list[$i]['images'][]['url'] = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/woocommerce-placeholder-600x600.png';
            }
        }
    }

    public function quickSearch($keyword = '')
    {
        $this->quick_search_routine($keyword, true);
        $this->getQuickMeta();
        $this->getPrimaryImages();
        $this->compact();
        $this->setDefaultImage();
        return array('products' => $this->list, 'count' => count($this->id_list));
    }
    public function search_products_by_vendor($keyword = '', $vendor_id = '', $page = 1, $count = 12){
        $this->search_routine_products_by_vendor($keyword,$vendor_id,$page,$count);
        $this->getQuickMeta();
        $this->getMeta();
        $this->getCategories(true);
        $this->getVendorData();
        $this->compact();
        $this->setDefaultImage();
        $this->FilterData();
        $this->generateFilterData();
        if((($page - 1) * $count) > count($this->list)){
            $page=1;
        }
        return array('products' => array_slice($this->list, ($page - 1) * $count, $count), 'count' => count($this->list), 'filter' => $this->filter);

    }
    private function compact()
    {
        $parent = [];
        $var = [];
        foreach ($this->full_list as $key => $value) {
            if (isset($value['parent']) && $value['parent'] != '' && in_array($value['parent'], $this->id_list)) {
                $p = $value['parent'];
                unset($value['parent']);
                $var[$p][] = $value;
            } else {
                $parent[$key][] = $value;
            }
        }
        $this->full_list = [];
        foreach ($this->attr_list as $key => $value) {
            $list = [];
            foreach ($value as $k => $v) {
                array_push($list, $v);
            }
            $parent[$key][0]['attributes'] = $list;
        }
        $this->attr_list = [];
        foreach ($var as $key => $value) {
            $parent[$key][0]['variations'] = $value;
        }
        $var = [];
        foreach ($parent as $key => $value) {
            if ($value[0]['name'] != '') {
                array_push($this->list, $value[0]);
            }
        }
        $parent = [];
    }

    private function setViewed(int $product_id)
    {
        
        $this->db->insert('products_views_detailed', array('product_id' => $product_id));
        $data = $this->db->search('products_views', array('product_id' => $product_id), 'AND', 'DEFAULT_WHERE', 1);
        if ($data['cant_rows'] == 0) {
            $this->db->insert('products_views', array('product_id' => $product_id, 'views' => 1));
        } else {
            $this->db->update('products_views', array('product_id' => $product_id), array('views' => $data[0]['views'] + 1));
        }
    }

    private function setKeywordSearch($keyword, $results = 0)
    {
        $this->db->insert('keywords_reserch', array('user_id' => $this->user_id, 'ip' => $_SERVER['REMOTE_ADDR'], 'keyword' => $keyword, 'results' => $results));
    }

    private function getVendorData()
    {
/*SELECT
        OvMdW_users.ID,
        OvMdW_users.user_nicename,
        OvMdW_usermeta.meta_value,
        (SELECT
           count(*)
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = '_dmce_operation_key' AND
            OvMdW_usermeta.user_id  = 6002 and 
        	(OvMdW_usermeta.meta_value IS NOT NULL AND OvMdW_usermeta.meta_value != '')) as has_dmce
        FROM
        OvMdW_users
        INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
        WHERE
        (OvMdW_usermeta.meta_key = 'dokan_store_name'  ) AND
        OvMdW_usermeta.user_id = 6002 AND
        OvMdW_usermeta.meta_value IS NOT NULL;*/
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
        OvMdW_users.ID,
        OvMdW_users.user_nicename,
        OvMdW_usermeta.meta_value,
        (SELECT
           (".count(array_unique(array_values($this->vendors)))." - count(*))
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'DMCE_key' AND
            OvMdW_usermeta.user_id IN (" . implode(",", array_unique(array_values($this->vendors))) . ")  AND
        	(OvMdW_usermeta.meta_value IS NOT NULL AND OvMdW_usermeta.meta_value != '')) AS dont_have_dmce
        FROM
        OvMdW_users
        INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
        WHERE
        OvMdW_usermeta.meta_key = 'dokan_store_name' AND
        OvMdW_usermeta.user_id IN (" . implode(",", array_values($this->vendors)) . ")           
        "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        //die(var_dump($query));
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            foreach ($this->vendors as $key => $value) {
                if ($value == $data[$i]['ID']) {
                    $this->full_list[$key]['vendor'] = $data[$i]['meta_value'];
                    $this->full_list[$key]['vendor_slug'] = $data[$i]['user_nicename'];
                    $this->full_list[$key]['vendor_id'] = $data[$i]['ID'];
                    $this->full_list[$key]['dont_have_dmce'] = $data[$i]['dont_have_dmce'];
                }
            }
        }

    }

    private function getChildren()
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_posts.ID,
            OvMdW_posts.post_author,
            OvMdW_posts.post_title,
            OvMdW_posts.post_status,
            OvMdW_posts.post_type,
            OvMdW_posts.post_name,
            OvMdW_posts.post_date,
            OvMdW_posts.post_modified,
            OvMdW_posts.post_parent,
            (SELECT
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_variation_description' AND
            OvMdW_postmeta.post_id = OvMdW_posts.ID LIMIT 1) AS post_excerpt
            FROM
            OvMdW_posts
            WHERE
            OvMdW_posts.post_parent IN (" . implode(",", $this->id_list) . ") AND
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product_variation'
            GROUP BY
            ovmdw_posts.ID            
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $key = $data[$i]['ID'];
            $this->full_list[$key]['product_id'] = $data[$i]['ID'];
            $this->full_list[$key]['name'] = $data[$i]['post_title'];
            $this->full_list[$key]['slug'] = $data[$i]['post_name'];
            $this->full_list[$key]['status'] = $data[$i]['post_status'];
            $this->full_list[$key]['description'] = $data[$i]['post_excerpt'];
            $this->full_list[$key]['short_description'] = $data[$i]['post_excerpt'];
            $this->full_list[$key]['created_at'] = $data[$i]['post_date'];
            $this->full_list[$key]['modified_at'] = $data[$i]['post_modified'];
            $this->full_list[$key]['parent'] = $data[$i]['post_parent'];
            $this->full_list[$key]['images'] = [];
        }
        unset($data);
        unset($query);
    }

    public function getOrderedCategories($save = false)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_terms.term_id,
            ovmdw_terms.`name`,
            ovmdw_terms.slug,
            ovmdw_term_taxonomy.taxonomy,
            ovmdw_term_relationships.object_id
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            WHERE
            ovmdw_term_relationships.object_id IN (" . implode(",", $this->id_list) . ") AND
            OvMdW_term_taxonomy.taxonomy = 'product_cat' AND
            OvMdW_term_taxonomy.parent = 0
            ORDER BY
            OvMdW_term_taxonomy.count DESC
            LIMIT 1           
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        $arr_cats = [];
        $this->storeTaxonomiesList($data, $arr_cats);
        if ($data['cant_rows'] > 0) {
            $this->getChildCategory($data[0]['term_id'], $arr_cats);
        }
        unset($data);
        unset($query);
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_terms.term_id,
            ovmdw_terms.`name`,
            ovmdw_terms.slug,
            ovmdw_term_taxonomy.taxonomy,
            ovmdw_term_relationships.object_id
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            WHERE
            ovmdw_term_relationships.object_id IN (" . implode(",", $this->id_list) . ") AND
            (ovmdw_term_taxonomy.taxonomy = 'product_tag')
            ORDER BY
            OvMdW_term_taxonomy.count DESC           
            "
        );
        $data = $this->db->plainQuery($query);
        $this->storeTaxonomiesList($data, $arr_cats);
        unset($data);
        unset($query);

        if ($save) {
            if ($this->user_id != 0) {
                $buy = new BuyTendency($this->user_id);
                $buy->updateData(array_unique($arr_cats));
            }
        }
        unset($data);
        unset($arr_cats);
        unset($query);
    }

    private function getChildCategory($parent, &$arr_cats)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_terms.term_id,
            ovmdw_terms.`name`,
            ovmdw_terms.slug,
            ovmdw_term_taxonomy.taxonomy,
            ovmdw_term_relationships.object_id
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            WHERE
            ovmdw_term_relationships.object_id IN (" . implode(",", $this->id_list) . ") AND
            OvMdW_term_taxonomy.taxonomy = 'product_cat' AND
            OvMdW_term_taxonomy.parent = " . $parent . "
            ORDER BY
            OvMdW_term_taxonomy.count DESC
            "
        );
        $data = $this->db->plainQuery($query);
        if ($data['cant_rows'] <= 0) {
            return false;
        }
        $this->storeTaxonomiesList($data, $arr_cats);
        return $this->getChildCategory($data[0]['term_id'], $arr_cats);
    }

    private function storeTaxonomiesList($data, &$arr_cats)
    {
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $prod = $data[$i]['object_id'];
            if ($this->full_list[$prod]) {
                if ($data[$i]['taxonomy'] == 'product_cat') {
                    array_push($this->full_list[$prod]['categories'], array(
                        'id' => $data[$i]['term_id'],
                        'name' => $data[$i]['name'],
                        'slug' => $data[$i]['slug'],
                    ));
                    array_push($arr_cats, $data[$i]['slug']);
                } else {
                    array_push($this->full_list[$prod]['tags'], array(
                        'id' => $data[$i]['term_id'],
                        'name' => $data[$i]['name'],
                        'slug' => $data[$i]['slug'],
                    ));
                }
            }
        }
    }

    private function getCategories($save = false)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_terms.term_id,
            ovmdw_terms.`name`,
            ovmdw_terms.slug,
            ovmdw_term_taxonomy.taxonomy,
            ovmdw_term_relationships.object_id
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            WHERE
            ovmdw_term_relationships.object_id IN (" . implode(",", $this->id_list) . ") AND
            (ovmdw_term_taxonomy.taxonomy = 'product_tag' OR
            ovmdw_term_taxonomy.taxonomy = 'product_cat')
            ORDER BY
            OvMdW_term_taxonomy.count DESC           
            "
        );
        $data = $this->db->plainQuery($query);
        $arr_cats = [];
        $this->storeTaxonomiesList($data, $arr_cats);
        if ($save) {
            if ($this->user_id != 0) {
                $buy = new BuyTendency($this->user_id);
                $buy->updateData(array_unique($arr_cats));
            }
        }
        unset($data);
        unset($arr_cats);
        unset($query);
    }

    private function getPrimaryImages()
    {
        if(count(array_unique(array_filter(array_keys($this->queneu_list))))==0){
            return false;
        }
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.post_id IN (" . implode(",", array_unique(array_filter(array_keys($this->queneu_list)))) . ") AND
            ovmdw_postmeta.meta_key = '_wp_attached_file'
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        $data = $this->db->plainQuery($query);
        /* for ($i = 0; $i < $data['cant_rows']; $i++) {
            $type = $this->queneu_list[$data[$i]['post_id']]['type'];
            $product_id = $this->queneu_list[$data[$i]['post_id']]['product_id'];
            if (!isset($this->full_list[$product_id]['images'])) {
                $this->full_list[$product_id]['images'] = [];
            }
            $url = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . str_replace('-scaled','',$data[$i]['meta_value']);
            if ($type == '_thumbnail_id') {
                array_unshift($this->full_list[$product_id]['images'], array('url' => $url));
            } else {
                array_push($this->full_list[$product_id]['images'], array('url' => $url));
            }
        } */

        foreach ($this->queneu_list as $attach_id => $product) {
            foreach ($product as $product_id => $type) {
                //echo $type.'--';
                //echo $product_id.'--/--';
                if (!isset($this->full_list[$product_id]['images'])) {
                    $this->full_list[$product_id]['images'] = [];
                }
                for ($i = 0; $i < $data['cant_rows']; $i++) {
                    if ($attach_id == $data[$i]['post_id']) {
                        //echo 'exist---/---';
                        $url = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . str_replace('-scaled', '', $data[$i]['meta_value']);
                        if ($type == '_thumbnail_id') {
                            array_unshift($this->full_list[$product_id]['images'], array('url' => $url));
                        } else {
                            array_push($this->full_list[$product_id]['images'], array('url' => $url));
                        }
                        $i = $data['cant_rows'];
                    }
                }
            }
        }

        /* for ($i = 0; $i < $data['cant_rows']; $i++) {
            $type = $this->queneu_list[$data[$i]['post_id']]['type'];
            $product_id = $this->queneu_list[$data[$i]['post_id']]['product_id'];
            if (!isset($this->full_list[$product_id]['images'])) {
                $this->full_list[$product_id]['images'] = [];
            }
            $url = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . str_replace('-scaled','',$data[$i]['meta_value']);
            if ($type == '_thumbnail_id') {
                array_unshift($this->full_list[$product_id]['images'], array('url' => $url));
            } else {
                array_push($this->full_list[$product_id]['images'], array('url' => $url));
            }
        } */
        //echo json_encode($this->queneu_list);
        //echo json_encode($data);
        unset($data);
        unset($query);

        //die();
    }

    private function manageMeta($meta_data)
    {
        $id = $meta_data['post_id'];
        switch ($meta_data['meta_key']) {
            case '_dokan_wholesale_meta':
                $m = new MetaValues();
                $dt =  $m->maybe_unserialize($meta_data['meta_value']);
                $this->full_list[$id]['price'] = $this->handle_values($dt['price']);
                $this->full_list[$id]['moq'] = $dt['quantity'];
                break;
            case '_product_attributes':
                $m = new MetaValues();
                $dt =  $m->maybe_unserialize($meta_data['meta_value']);
                if (is_array($dt)) {
                    foreach ($dt as $key => $value) {
                        if (!isset($this->attr_list[$id][$key]['name'])) {
                            $this->attr_list[$id][$key]['name'] = $this->handle_values($value['name']);
                            $this->attr_list[$id][$key]['unique_name'] = str_replace(['Ã±', ' '], ['n', '-'], strtolower($value['name']));
                            $this->attr_list[$id][$key]['is_variation'] = $this->handle_values($value['is_variation']);
                            $this->attr_list[$id][$key]['value'] = array_map('trim', explode('|', $value['value']));
                            $this->attr_list[$id][$key]['type'] = 'regular';
                            $this->attr_list[$id][$key]['display_value'] = [];
                        }
                    }
                }
                break;
            case '_custom_attr_id':
                $m = new MetaValues();
                $dt =  $m->maybe_unserialize($meta_data['meta_value']);
                if (is_array($dt)) {
                    foreach ($dt as $key => $value) {
                        $query = str_replace(
                            'ovmdw_',
                            TABLE_PREFIX,
                            "SELECT
                        ovmdw_BI_custom_attribute_field.custom_attr_name,
                        ovmdw_BI_custom_attribute_field.custom_attr_type,
                        ovmdw_BI_custom_attribute_meta.custom_ref_post,
                        ovmdw_BI_custom_attribute_field.custom_attr_display_name,
                        ovmdw_BI_custom_attribute_meta.custom_attr_key,
                        ovmdw_BI_custom_attribute_meta.custom_attr_value
                        FROM
                        ovmdw_BI_custom_attribute_field
                        INNER JOIN ovmdw_BI_custom_attribute_meta ON ovmdw_BI_custom_attribute_meta.custom_attr_id = ovmdw_BI_custom_attribute_field.custom_attr_id
                        WHERE
                        ovmdw_BI_custom_attribute_field.custom_attr_id = '" . $key . "'"
                        );
                        //die($query);
                        $attr = $this->db->plainQuery($query);
                        for ($i = 0; $i < $attr['cant_rows']; $i++) {
                            $name = $attr[$i]['custom_attr_name'];
                            $name_fix = str_replace(['Ã±', ' '], ['n', '-'], strtolower($name));
                            $ref = $attr[$i]['custom_ref_post'];
                            $this->attr_list[$id][$name]['type'] = $attr[$i]['custom_attr_type'];
                            $this->attr_list[$id][$name]['is_variation'] = 1;
                            $this->attr_list[$id][$name]['name'] = ($attr[$i]['custom_attr_display_name'] != '') ? $attr[$i]['custom_attr_display_name'] : $name_fix;
                            $this->attr_list[$id][$name]['unique_name'] = $name_fix;
                            switch ($attr[$i]['custom_attr_type']) {
                                case 'Color':
                                    if ($attr[$i]['custom_attr_key'] == 'meta_name_custom_attr') {
                                        $this->attr_list[$id][$name]['display_value'][$ref]['name'] = $attr[$i]['custom_attr_value'];
                                    }
                                    if ($attr[$i]['custom_attr_key'] == 'meta_color_custom_attr') {
                                        $this->attr_list[$id][$name]['display_value'][$ref]['value'] = $attr[$i]['custom_attr_value'];
                                    }
                                    break;
                                case 'Imagen':
                                    if ($attr[$i]['custom_attr_key'] == 'meta_name_custom_attr') {
                                        $this->attr_list[$id][$name]['display_value'][$ref]['name'] = $attr[$i]['custom_attr_value'];
                                    }
                                    if ($attr[$i]['custom_attr_key'] == 'meta_url_custom_attr') {
                                        $this->attr_list[$id][$name]['display_value'][$ref]['value'] = $attr[$i]['custom_attr_value'];
                                    }
                                    break;
                                default:
                                    if ($attr[$i]['custom_attr_key'] == 'meta_name_custom_attr') {
                                        $this->attr_list[$id][$name]['display_value'][$ref]['name'] = $attr[$i]['custom_attr_value'];
                                    }
                                    break;
                            }
                        }
                        unset($attr);
                        unset($query);
                    }
                }
                //die(json_encode($this->attr_list));

                break;
            case '_product_image_gallery':
                $items = explode(',', $meta_data['meta_value']);
                foreach ($items as $value) {
                    //$this->queneu_list[$value] = array('product_id' => $id, 'type' => '_product_image_gallery');
                    $this->queneu_list[$value][$id] = '_product_image_gallery';
                }
                break;
            case '_thumbnail_id':
                $items = explode(',', $meta_data['meta_value']);
                foreach ($items as $value) {
                    //$this->queneu_list[$value] = array('product_id' => $id, 'type' => '_thumbnail_id');
                    $this->queneu_list[$value][$id] = '_thumbnail_id';
                }
                break;

            default:
                $field = $this->meta_fields[$meta_data['meta_key']];
                $this->full_list[$id][$field] = $this->handle_values($meta_data['meta_value']);
                break;
        }
    }

    private function getMeta($quick = false)
    {
        $keys = ($quick) ? $this->minimun_meta_fields : $this->meta_fields;
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_key,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.post_id IN (" . implode(',', array_keys($this->full_list)) . ")
            AND
            ovmdw_postmeta.meta_key IN ('" . implode("','", array_keys($keys)) . "')
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->manageMeta($data[$i]);
        }
        unset($data);
        unset($query);
    }

    private function loadAttributes()
    {
        foreach ($this->full_list as $key => $value) {
            if ($value['parent'] != 0) {
                $this->full_list[$key]['attributes'] = $this->getChildrenAttr($key);
            }
        }
    }

    private function getChildrenAttr($product_id = 0)
    {
        $arr = [];
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_postmeta.post_id,
            OvMdW_postmeta.meta_key,
            OvMdW_postmeta.meta_value
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.post_id = '" . $product_id . "' AND
            OvMdW_postmeta.meta_key LIKE 'attribute_%'           
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $item['name'] = str_replace('attribute_', '', $data[$i]['meta_key']);
            $item['value'] = $data[$i]['meta_value'];
            array_push($arr, $item);
        }
        return $arr;
    }


    private function getQuickMeta()
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_key,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.post_id IN (" . implode(',', array_keys($this->full_list)) . ")
            AND
            (ovmdw_postmeta.meta_key = '_thumbnail_id' OR ovmdw_postmeta.meta_key = '_product_image_gallery')
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->manageMeta($data[$i]);
        }
        unset($data);
        unset($query);
    }

    private function handle_values($value)
    {
        $m = new MetaValues();
        $dt =  $m->maybe_unserialize($value);
        unset($m);
        return $dt;
    }


    private function findProduct($keyword = '')
    {
        if ($keyword == '' || $keyword == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            AND 
                ovmdw_posts.post_status = 'publish'
            AND 
                ovmdw_posts.post_name = '" . $this->db->sanitizeData($keyword) . "'
            GROUP BY 
            ovmdw_posts.ID
            LIMIT 1"
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
            $this->setViewed($data[$i]['ID']);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }


    private function findProductByIds(array $ids, bool $force = false)
    {
        if (!$ids || !is_array($ids)) {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            
            AND 
            ovmdw_posts.ID IN (" . implode(',', $ids) . ")
            GROUP BY
            ovmdw_posts.ID
            HAVING
            
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_title ASC,
                ovmdw_posts.post_excerpt ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query, force:$force);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function search_routine_last($number = 60, $page = 1, $count = 12)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            GROUP BY
            ovmdw_posts.ID
            HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_date DESC,
                ovmdw_posts.post_title ASC,
                ovmdw_posts.post_excerpt ASC
            LIMIT " . $this->db->sanitizeData($number) . "
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_routine($keyword = '', $page = 1, $count = 12)
    {
        if ($keyword == '' || $keyword == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $keyword = str_replace(' ', '%', trim($keyword));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            AND (
                ovmdw_posts.post_title LIKE '%" . $this->db->sanitizeData($keyword) . "%' OR
                ovmdw_posts.post_excerpt LIKE '%" . $this->db->sanitizeData($keyword) . "%'
                OR ovmdw_posts.ID IN (
                    (
                        SELECT
                            ovmdw_postmeta.post_id
                        FROM
                            ovmdw_postmeta
                        WHERE
                            (
                                ovmdw_postmeta.meta_key = '_custom_product_brand'
                                AND ovmdw_postmeta.meta_value LIKE '" . $this->db->sanitizeData($keyword) . "%'
                            )
                        OR (
                            ovmdw_postmeta.meta_key = '_custom_product_model'
                            AND ovmdw_postmeta.meta_value LIKE '" . $this->db->sanitizeData($keyword) . "%'
                        )
                        OR (
                            ovmdw_postmeta.meta_key = '_custom_product_arancel'
                            AND ovmdw_postmeta.meta_value LIKE '" . $this->db->sanitizeData($keyword) . "%'
                        )
                        GROUP BY
                            ovmdw_postmeta.post_id
                    )
                )
                OR ovmdw_posts.ID IN (
                    (
                        SELECT
                            ovmdw_term_relationships.object_id AS ID
                        FROM
                            ovmdw_term_relationships
                        INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
                        INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
                        WHERE
                            (
                                ovmdw_term_taxonomy.taxonomy = 'product_cat'
                                OR ovmdw_term_taxonomy.taxonomy = 'product_tag'
                            )
                        AND ovmdw_terms.`name` LIKE '%" . $this->db->sanitizeData($keyword) . "%'
                        GROUP BY
                            ovmdw_term_relationships.object_id
                    )
                )
                OR ovmdw_posts.post_author IN (
                    (
                        SELECT
                        OvMdW_users.ID
                        FROM
                        OvMdW_users
                        INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
                        WHERE
                        OvMdW_usermeta.meta_key = 'dokan_store_name' AND
                        OvMdW_usermeta.meta_value LIKE '%" . $this->db->sanitizeData($keyword) . "%'
                        GROUP BY
                        OvMdW_users.ID
                    )
                )
                
            )
            GROUP BY
            ovmdw_posts.ID
            HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_title ASC,
                ovmdw_posts.post_excerpt ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->setKeywordSearch($keyword, $data['cant_rows']);
        //$this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_routine_by_vendor($vendor_id = '', $page = 1, $count = 12)
    {
        if ($vendor_id == '' || $vendor_id == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_id = trim($vendor_id);

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            AND ovmdw_posts.post_author = '" . $this->db->sanitizeData($vendor_id) . "'
            GROUP BY
            ovmdw_posts.ID
            HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_title ASC,
                ovmdw_posts.post_excerpt ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_category_routine($keyword = '', $page = 1, $count = 12)
    {
        if ($keyword == '' || $keyword == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $key = explode('|', $keyword);
        $search = "'" . implode("','", $this->db->sanitizeData($key)) . "'";

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            INNER JOIN ovmdw_posts ON ovmdw_posts.ID = ovmdw_term_relationships.object_id
            WHERE
            ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            AND
                (
                    (
                    ovmdw_term_taxonomy.taxonomy = 'product_cat'
                    )
                    AND ovmdw_terms.`slug` IN (" . $search . ")
                )
                OR
                (
                    ovmdw_terms.term_id IN (
                        SELECT
                        ovmdw_terms.term_id
                        FROM
                            `ovmdw_terms`
                        WHERE
                            taxonomy = 'product_cat'
                            AND ovmdw_terms.`slug` IN (" . $search . ")
                            GROUP BY
                            ovmdw_terms.term_id
                    )
                )
                GROUP BY
                ovmdw_term_relationships.object_id
                HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_title ASC,
                ovmdw_posts.post_excerpt ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }


    private function random_by_cat($cat, $qty)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
        ovmdw_posts.ID,
        ovmdw_posts.post_author,
        ovmdw_posts.post_title,
        ovmdw_posts.post_excerpt,
        ovmdw_posts.post_status,
        ovmdw_posts.post_type,
        ovmdw_posts.post_name,
        ovmdw_posts.post_date,
        ovmdw_posts.post_modified
        FROM
        ovmdw_term_relationships
        INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
        INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
        INNER JOIN ovmdw_posts ON ovmdw_posts.ID = ovmdw_term_relationships.object_id
        WHERE
            ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            AND
            (
                (
                ovmdw_term_taxonomy.taxonomy = 'product_cat'
                )
                AND ovmdw_terms.`slug` IN ('" . $cat . "')
            )
            OR
            (
                ovmdw_terms.term_id IN (
                    SELECT
                    ovmdw_terms.term_id
                    FROM
                        `ovmdw_terms`
                    WHERE
                        taxonomy = 'product_cat'
                        AND ovmdw_terms.`slug` IN ('" . $cat . "')
                        GROUP BY
                        ovmdw_terms.term_id
                )
            )
            GROUP BY
            ovmdw_term_relationships.object_id
            HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
        ORDER BY
            ovmdw_posts.post_title ASC,
            ovmdw_posts.post_excerpt ASC
        "
        );
        $data = $this->db->plainQuery($query);
        if ($data['cant_rows'] == 0) {
            return false;
        }
        if ($data['cant_rows'] <= $qty) {
            for ($i = 0; $i < $data['cant_rows']; $i++) {
                $this->fillPrimaryList($data, $i);
            }
        } else {
            for ($loop = 0; $loop < $qty; $loop++) {
                $i = $this->getRandomIndexVerified($data['cant_rows'], $data);
                $this->fillPrimaryList($data, $i);
            }
        }
        unset($data);
        unset($query);
        return;
    }

    private function fillPrimaryList($data, $i)
    {
        $key = $data[$i]['ID'];
        $this->id_list[$key]['product_id'] = $data[$i]['ID'];
        $this->id_list[$key]['name'] = $data[$i]['post_title'];
        $this->id_list[$key]['slug'] = $data[$i]['post_name'];
        $this->id_list[$key]['status'] = $data[$i]['post_status'];
        $this->id_list[$key]['description'] = str_replace("\r\n\r\n",'<br />',$data[$i]['post_excerpt'] == '' ? $data[$i]['post_content']:$data[$i]['post_excerpt']);
        $this->id_list[$key]['short_description'] = str_replace("\r\n\r\n",'<br />',$data[$i]['post_excerpt']);
        $this->id_list[$key]['created_at'] = $data[$i]['post_date'];
        $this->id_list[$key]['modified_at'] = $data[$i]['post_modified'];
        $this->id_list[$key]['variations'] = [];
        $this->id_list[$key]['attributes'] = [];
        $this->id_list[$key]['categories'] = [];
        $this->id_list[$key]['tags'] = [];
        $this->id_list[$key]['images'] = [];

        $this->vendors[$key] = $data[$i]['post_author'];
    }

    private function getRandomIndexVerified($max, $data = array())
    {
        if ($max == 0 || count($data) == 0) {
            return false;
        }
        $i = random_int(0, $max);
        //echo $i.'-'.$data[$i]['ID'].' ';
        $id = $data[$i]['ID'];
        if ($id == '' || isset($this->id_list[$id])) {
            //echo $i.'-'.$data[$i]['ID'].' exist'.' ';
            return $this->getRandomIndexVerified($max, $data);
        }
        return $i;
    }

    private function search_random_routine($count = 2)
    {
        $this->list = [];
        $this->id_list = [];
        $this->full_list = [];
        $categories = new Categories();
        $cats = $categories->getAllData(false)->getCategories();
        foreach ($cats as $key => $value) {
            $this->random_by_cat($value['slug'], $count);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        return $this;
    }



    private function quick_search_routine($keyword = '', $quick = false)
    {
        if ($keyword == '' || $keyword == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $keyword = str_replace(' ', '%', trim($keyword));
        $limit = ($quick) ? 'LIMIT 0,5' : '';

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type = 'product'
            AND ovmdw_posts.post_status = 'publish'
            AND (
                ovmdw_posts.post_title LIKE '%" . $this->db->sanitizeData($keyword) . "%'
                OR ovmdw_posts.ID IN (
                    (
                        SELECT
                            ovmdw_postmeta.post_id
                        FROM
                            ovmdw_postmeta
                        WHERE
                            (
                                ovmdw_postmeta.meta_key = '_custom_product_brand'
                                AND ovmdw_postmeta.meta_value LIKE '" . $this->db->sanitizeData($keyword) . "%'
                            )
                        GROUP BY
                            ovmdw_postmeta.post_id
                            
                    )
                )
                OR ovmdw_posts.ID IN (
                    (
                        SELECT
                            ovmdw_term_relationships.object_id AS ID
                        FROM
                            ovmdw_term_relationships
                        INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
                        INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
                        WHERE
                            (
                                ovmdw_term_taxonomy.taxonomy = 'product_cat'
                            )
                        AND ovmdw_terms.`name` LIKE '" . $this->db->sanitizeData($keyword) . "%'
                        GROUP BY
                            ovmdw_term_relationships.object_id
                        
                    )
                )
                
            )
            GROUP BY
            ovmdw_posts.ID
            HAVING
            ovmdw_posts.post_status = 'publish' AND
            ovmdw_posts.post_type = 'product' AND
            OvMdW_posts.ID NOT IN (SELECT
            OvMdW_postmeta.post_id
            FROM
            OvMdW_postmeta
            WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND
            OvMdW_postmeta.meta_value = 'hidden'
            GROUP BY
            OvMdW_postmeta.post_id
            )
            ORDER BY
                ovmdw_posts.post_title ASC
            " . $limit . ""
        );
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function search_routine_products_by_vendor($keyword, $vendor_id ,$page =1,$count = 12){
        if ($vendor_id == '' || $vendor_id == ' ' ) {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }
        if ($keyword == '' || $keyword == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_id = trim($vendor_id);
        $keyword = str_replace(' ', '%', trim($keyword));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ID,
            post_author,
            post_title,
            post_excerpt,
            post_status,
            post_type,
            post_name,
            post_date,
            post_modified,
            post_content
            FROM
            `OvMdW_posts`
            WHERE
                post_type = 'product'
            AND post_author = '". $this->db->sanitizeData($vendor_id) ."'
            AND (
                post_title LIKE '%". $this->db->sanitizeData($keyword) ."%'
                OR ID IN (
                    (
                        SELECT
                            post_id
                        FROM
                            `OvMdW_postmeta`
                        WHERE
                            
                         (
                            meta_key = '_sku'
                            AND meta_value LIKE '". $this->db->sanitizeData($keyword) ."%'
                        )
                        GROUP BY
                            post_id
                    )
                )
            )
            GROUP BY
            ID
            HAVING
            post_type = 'product' 
            ORDER BY
                post_title ASC,
                post_excerpt ASC;
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        //die(var_dump($data));
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
    }
    public function deleteOne($id, $dokan_sale_meta){
        return $this->deleteDokanWholesaleColumn($id, $dokan_sale_meta,2);
     }
     public function updateAuthor($productId,$newAuthorId , $limit = 1 ){
         $query = str_replace(
             'ovmdw_',
             TABLE_PREFIX,
             "UPDATE OvMdW_posts SET post_author = '".$newAuthorId."' WHERE id=".$productId." LIMIT ".$limit.";   
             "
         );
         $data = $this->db->plainQuery($query);
         //die();
         return $data;
     }
     private function deleteDokanWholesaleColumn($id,$dokan_sale_meta, $limit ='1'){
         
         $query = str_replace(
             'ovmdw_',
             TABLE_PREFIX,
             "UPDATE  OvMdW_postmeta set meta_value = '".serialize($dokan_sale_meta)."' WHERE meta_key='_dokan_wholesale_meta' AND post_id = '".$id."' LIMIT ".$limit.";   
             "
         );
         $data = $this->db->plainQuery($query);
         //die($query);
         return $data;
     }
     public function createRelationship($post_id, $term){
      $this->createRelationshipRoutine($post_id, $term);
     }
     private function createRelationshipRoutine($post_id, $term){
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "INSERT  INTO OvMdW_term_relationships (object_id, term_taxonomy_id, term_order) VALUES ('".$post_id."',(SELECT `OvMdW_term_taxonomy`.`term_taxonomy_id` FROM `OvMdW_terms` INNER JOIN `OvMdW_term_taxonomy` ON `OvMdW_terms`.term_id = `OvMdW_term_taxonomy`.`term_id` WHERE name = '".$term."' LIMIT 1), 0);   
            "
        );
        $data = $this->db->plainQuery($query);
        //die($query);
        return $data;

     }
}
