<?php

class FCI_class implements Currier
{

    private $db;
    private Destination $destination;
    private Cart $cart;
    private Package $package;
    private array $aracelData = [];
    private $dai = 0;
    private $tax = 0;
    private $isc = 0;
    private $icc = 0;
    private $total = 0;
    public $totalTransitDays = 10;


    public $consularTax = 107.0;
    public $consularTaxOversize = 2000.0;
    public $minimun_logistic_cost = 20.0;
    public $percentage_logistic_cost = 0.14;
    public $percentage_FEE_cost = 0.05;
    public $minimun_FEE_cost = 15;
    public $itbms = 0.07;

    public $logistic_cost = 0;
    public $consularTax_cost = 0;
    public $shipping = 10.0;
    public $shippingOversize = 10.0;
    public $system = 3.0;

    public $error = '';
    public $response = [];
    public $status = true;

    public $pricing = [
        0 => [
            'min' => 0,
            'max' => 0.01728,
            'price' => 10
        ],
        1 => [
            'min' => 0.01729,
            'max' => 0.08,
            'price' => 15
        ],
        2 => [
            'min' => 0.08,
            'max' => 0.175,
            'price' => 35
        ],
        3 => [
            'min' => 0.175,
            'max' => 1,
            'price' => 40
        ],
    ];
    public $bank_fee = 0.03;
    public $bank_fee_cost = 0;

    function __construct()
    {
        $this->db = new collection;
    }

    function generateResponseSchema(array $data, bool $include_taxes = false)
    {

        if ($this->package->getTotalCBM()['ground'] > 1) {
            return [];
        }

        if($this->package->getTotalOrder()>2000){
            return [];
        }

        $res = [];
        $res['method_title'] = 'FCI Logistics';
        $res['description'] = 'FCI Logistics';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/fci.png';
        $res['message'] = 'Esta tarifa incluye el envío puerta a puerta en el área metropolitana de Ciudad de Panamá.  La entrega puerta a puerta fuera del área metropolitana o en una ciudad del interior, tiene un costo adicional el cual deberá ser asumido por el consignatario y este será comunicado por nuestro departamento logístico cuando la mercancía llegue a la ciudad.';
        $prod['productName'] = 'Envío Terrestre 7-10 días hábiles. Este plazo de entrega está sujeto a verificación de inventario por parte del vendedor y al plazo de entrega de proveedor logístico.';
        $prod['type'] = 'ground';
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = round($this->total, 2);
        $prod['volumetric'] = $this->package->getTotalCBM();
        $prod['deliveryCapabilities']['totalTransitDays'] = $this->totalTransitDays;
        $prod['details'][] = $this->insertDetail('Impuesto de Nacionalización', 'Cargos por nacionalización', $this->dai + $this->tax + $this->system + $this->consularTax_cost);
        //$prod['details'][] = $this->insertDetail('ITBMS', 'Impuestos nacionales', $this->tax);
        $prod['details'][] = $this->insertDetail('Trámite logístico', 'Trámite logístico', $this->logistic_cost);
        // if ($this->consularTax_cost > 0) {
        //     $prod['details'][] = $this->insertDetail('Tasa consular', '', $this->consularTax_cost);
        // }
        $prod['details'][] = $this->insertDetail('Envío', '', round(($this->getShippingAmount()), 2));
        $prod['details'][] = $this->insertDetail('Comisión', '', round($this->bank_fee_cost, 2));
        $res['options'][] = $prod;
        return $res;
    }

    private function packageConsularFee():void {
        if($this->package->getTotalOrder()>$this->consularTaxOversize){
            $this->consularTax_cost = $this->consularTax;
        }
    }

    private function getBankFeeTotal(): float
    {
        return ($this->total + $this->package->getTotalOrder()) * $this->bank_fee;
    }

    private function calculateLogisticAmount()
    {
        $total = 0;
        if ($this->percentage_logistic_cost * $this->package->getTotalOrder() < $this->minimun_logistic_cost) {
            $total += ($this->minimun_logistic_cost + ($this->minimun_logistic_cost * $this->itbms));
        } else {
            $total += ($this->percentage_logistic_cost * $this->package->getTotalOrder()) + (($this->percentage_logistic_cost * $this->package->getTotalOrder()) * $this->itbms);
        }
        return $total;
    }

    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }

    public function errorResponse()
    {
        $res['error']=true;
        $res['method_title'] = 'FCI Logistics';
        $res['description'] = 'FCI Logistics';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/fci.png';
        $res['message']=$this->error;
        $prod['type']='ground';
        $prod['productName']=$this->error;
        $prod['totalPrice']['priceCurrency']='USD';
        $prod['totalPrice']['price']=0;
        $prod['volumetric']=$this->package->getTotalCBM();
        $prod['deliveryCapabilities']['totalTransitDays']=0;
        $res['options'][]=$prod;
        return $res;
    }

    private function loadArancelDetails(): void
    {
        $arancelList = [];
        foreach ($this->package->getItems() as $item) {
            if ($item instanceof PackageItem) {
                array_push($arancelList, $item->get_product_arancel());
            }
        }
        $data = $this->db->plainQuery("SELECT *  From ovmdw_dmce_hs_codes_complete WHERE dmce_hs_codes IN (" . implode(',', $arancelList) . ")");
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->aracelData[$data[$i]['dmce_hs_codes']] = $data[$i];
        }
    }


    public function calculateDai(PackageItem $item)
    {
        $value = 0;
        if (isset($this->aracelData[$item->get_product_arancel()])) {
            $value = ($this->aracelData[$item->get_product_arancel()]['dai'] / 100) * $item->getTotalAmount();
            $this->dai += $value;
        }
        return $value;
    }

    public function calculateTax(PackageItem $item, float $dai)
    {
        $value = 0;
        if (isset($this->aracelData[$item->get_product_arancel()])) {
            $value = ($this->aracelData[$item->get_product_arancel()]['itbms'] / 100) * (($item->getTotalAmount()) + $dai);
            $this->tax += $value;
        }
        return $value;
    }

    public function calculateISC(PackageItem $item)
    {
        $value = 0;

        return $value;
    }

    public function calculateICC(PackageItem $item)
    {
        $value = 0;

        return $value;
    }

    private function CalculateShipping(): void
    {
        foreach ($this->package->getItems() as $item) {
            if ($item instanceof PackageItem) {
                $dai = $this->calculateDai($item);
                //$isf=$this->calculateIcs($item);
                //$icc=$this->calculateIcc($item);
                $tax = $this->calculateTax($item, $dai);
            }
        }
    }

    private function recalculate()
    {
        $this->total = 0;
        $this->logistic_cost = $this->calculateLogisticAmount();
        $this->packageConsularFee();
        $this->total += ($this->dai + $this->tax + $this->system);
        $this->total += $this->logistic_cost;
        //$this->total += $this->consularTax_cost;
        $this->total += $this->getShippingAmount();
        $this->bank_fee_cost = $this->getBankFeeTotal();
        $this->total += $this->bank_fee_cost;
        $this->total = round($this->total, 2);
    }

    private function getShippingAmount(): float
    {
        foreach ($this->pricing as $price) {
            if ($this->package->getTotalCBM()['ground'] <= $price['max']) {
                return $price['price'];
            }
        }
    }

    function checkout_rate(Destination $_item, Package $package, bool $include_taxes): array
    {
        $this->destination = $_item;
        $this->package = $package;

        if ($this->package->getTotalCBM()['ground'] > 1) {
            return [];
        }

        if($this->package->getTotalOrder()>2000){
            return [];
        }
        $this->loadArancelDetails();
        $this->CalculateShipping();
        $this->recalculate();
        if ($this->total != 0) {
            return $this->generateResponseSchema([], $include_taxes);
        }
       
        return [];
    }
}
