<?php 

class TAM_Class implements Currier
{

    private $db;
    private Destination $destination;
    private Cart $cart;
    private Package $package;
    private $total = 0;

    public $volumetricWeightCosts = [
        'Honduras' => [
            'min' => 1,
            'cost' => 9,
            'bank_fee' => 0.03,
            'taxes' => []
        ]
    ];

    public $logistic_cost = 0;
    public $rates = null;
    public $shipping = 0;
    public $error = '';
    public $response = [];
    public $status = true;

    function __construct()
    {
        $this->db = new collection;
    }

    function generateResponseSchema(array $data, bool $include_taxes = false)
    {
        $res = [];

        $res['method_title'] = 'LSW';
        $res['description'] = 'LSW';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['message'] = 'Envío aéreo realizado bajo la modalidad DDP (entrega puerta a puerta). Entregas cubiertas hasta Tegucigalpa y San Pedro Sula, en caso de ser una dirección diferente el operador aduanal le contactará para coordinación de entrega una vez la mercancía llegue a destino, este costo no está cubierto en la tarifa ofertada.';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/lsw.png';
        $prod['type'] = 'air';
        $prod['productName'] = 'LSW';
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = round($this->total, 2);
        $prod['volumetric'] = $this->package->getTotalCBM()['air'];
        $prod['deliveryCapabilities']['totalTransitDays'] = 0;
        //$prod['details'][] = $this->insertDetail('Trámite logístico', 'Trámite logístico', $this->logistic_cost);
        //$prod['details'][] = $this->insertDetail('Impuesto de Nacionalización', 'Cargos por nacionalización', 0);
        $prod['details'][] = $this->insertDetail('Envío', '', $this->shipping);
        $res['options'][] = $prod;
        return $res;
    }

    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }

    public function errorResponse()
    {
        $res['error']=true;
        $res['method_title'] = 'LSW';
        $res['description'] = 'LSW';
        $res['include_taxes'] = '';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/lsw.png';
        $prod['productName'] = 'LSW';
        $res['message']=$this->error;
        $prod['type']='air';
        $prod['productName']=$this->error;
        $prod['totalPrice']['priceCurrency']='USD';
        $prod['totalPrice']['price']=0;
        $prod['volumetric']=$this->package->getTotalCBM()['air'];
        $res['options'][]=$prod;
    }

    private function recalculate()
    {
        $this->total = 0;
        $this->logistic_cost = $this->calculateLogisticAmount();
        $this->total += round($this->logistic_cost, 2);
        $this->total += round($this->shipping, 2);
    }

    function checkout_rate(Destination $_item, Package $package, bool $include_taxes): array
    {
        $this->destination = $_item;
        $this->package = $package;
        $this->recalculate();
        if ($this->total != 0) {
            return $this->generateResponseSchema([], $include_taxes);
        }
        return [];
    }

    private function calculateLogisticAmount()
    {
        $total = 0;
        if (isset($this->volumetricWeightCosts[$this->destination->country])) {
            $this->rates = $this->volumetricWeightCosts[$this->destination->country];
            
            foreach ($this->rates['taxes'] as $key => $value) {
                $total += $value['cost'];
            }
            $this->shipping = $this->getShippingPrice();
        }

        return $total;
    }

    private function packageHasMinimum():bool
    {
        return $this->package->getTotalCBM()['air'] > $this->rates['min'];
    }

    private function getShippingPrice(): float
    {
        $volumeWeight = $this->package->getTotalCBM()['air'];
        $sub_total = $volumeWeight * $this->rates['cost'];
        if(!$this->packageHasMinimum()){
            $sub_total = $this->rates['min'] * $this->rates['cost'];
        }
        return round($sub_total + $this->getBankFeeTotal($sub_total), 2);
    }

    private function getBankFeeTotal(float $sub_total): float
    {
        return round($sub_total * $this->rates['bank_fee'], 2);
    }

}