<?php

class Shipping
{

    private $db;
    private $providers;
    private Destination $destination;
    private Cart|Colibri_class $cart;
    private Package $package;
    private array $response;
    public array $errors = [];

    public function __construct()
    {
        $this->db = new collection();
    }

    /**
     * load registered shipping providers from database
     * 
     * @return void
     */
    public function registeredProviders()
    {
        $data = $this->db->plainQuery("
        SELECT
            bm_providers.id,
            bm_providers.tplCompanyRegNo,
            bm_providers.tplCompanyName,
            bm_providers.methodNo,
            bm_providers.class_name,
            bm_providers.include_taxes
        FROM
            bm_providers
        INNER JOIN rel_country_provider ON rel_country_provider.id_provider = bm_providers.id
        LEFT JOIN bi_country_table ON bi_country_table.id_country = rel_country_provider.id_country
        WHERE
            (bi_country_table.country_name = '" . $this->db->sanitizeData($this->destination->country) . "'
        OR rel_country_provider.id_country = 0) AND 
        bm_providers.id NOT IN(
        SELECT
        a.id_provider
        FROM
        rel_country_provider AS a
        INNER JOIN bi_country_table AS b ON b.id_country = a.exclude
        WHERE
        b.country_name = '" . $this->db->sanitizeData($this->destination->country) . "'
        GROUP BY
        a.id
        )
        GROUP BY
            bm_providers.id
        ");
        foreach ($data as $provider) {
            if ($provider['methodNo']) {
                if (class_exists($provider['class_name'])) {
                    $this->providers[$provider['methodNo']] = $provider;
                }
            }
        }
        return $this;
    }


    /**
     * upload data from json file
     * @param Array $data
     * 
     * @return void
     */
    public function uploadDataCheckOut(array $data)
    {
        if (!isset($data['address'])) {
            throw new Exception("Formato de dirección inválido", 2);
        }
        $this->setDestination($data['address']);
        if (!isset($data['package'])) {
            throw new Exception("Formato de envío inválido", 2);
        }
        $this->setPackage($data['package']);
        $this->registeredProviders();
        $this->response['providers'] = [];
        foreach ($this->providers as $key => $provider) {
            if ($provider['class_name'] != '' && class_exists($provider['class_name'])) {
                $currier = new $provider['class_name'];
                if ($currier instanceof Currier) {
                    $res = $currier->checkout_rate($this->destination, $this->package, $provider['include_taxes']);
                    if (!empty($res)) {
                        $res['method_id']=$provider['methodNo'];
                        $this->response['providers'][] = $res;
                        $this->response['items'] =$this->package->getItemsArray();
                    }
                    if (!$currier->status) {
                        $this->errors[$key] = $currier->error;
                    }
                    $pack=array(
                        'items'=>$this->package->getItemsArray(),
                        'totalItems'=>$this->package->getTotalItems(),
                        'CBM'=>$this->package->getTotalCBM(),
                        'Weight'=>$this->package->getTotalWeight(),
                        'errors'=>implode(',',$this->errors),
                    );
                    $this->db->insert('checkout_rate_log',array("package"=>serialize($pack),"destination"=>serialize($this->destination->toArray()),"date"=>date('Y-m-d H:i:s'),"provider"=>$provider['methodNo'],"response"=>serialize($res['options'])));
                    if(strcasecmp($provider['class_name'],'Dbs_class') == 0){

                    }
                }
            }
        }
        return $this;
    }

    /**
     * get response data from object as array
     * 
     * @return array
     */

    public function getResponse()
    {
        return $this->response;
    }


    /**
     * set current destination
     * 
     * @return void
     */
    function setDestination($destination)
    {
        if (!is_a($destination, 'Destination')) {
            try {
                $this->destination = new Destination($destination);
            } catch (\Throwable $th) {
                throw new Exception("Formato de dirección inválido", 2);
            }
        }
        return $this;
    }

    /**
     * generate package details from cart
     * 
     * @return void
     */
    private function getPackageDetails()
    {
        $this->package = new Package($this->cart);
        return $this;
    }


    /**
     * set package details from package info in Json format
     * 
     * @param Array $package
     * 
     * @return void
     */
    private function setPackage($package)
    {

        if (!isset($package['id_user']) || !isset($package['hash'] ) ) {
            throw new Exception("Formato de envío inválido o inexistente", 3);
        }
        try {
            if(isset($package["colibri_id"])){
                $this->cart = new Colibri_class($package['colibri_id']);
                $this->getPackageDetails();
                return;
            }
            $this->cart = new Cart($package['id_user'], $package['hash']);
            $this->getPackageDetails();
        } catch (\Throwable $th) {
            throw new Exception("Carrito invalido", 4);
        }
        return $this;
    }
}
