<?php

namespace BulkUpload\Factories;
use BulkUpload\Product as ProductObject;

/**
 * Product abstract class is used as a Factory model to be extended
 * from children that are represented as product, including variable 
 * and variation product. 
 */
abstract class Product
{

    abstract protected function insertProduct(ProductObject $product):void;

    abstract protected function insertProductInWordpress(ProductObject $product):void;

    abstract protected function insertProductInBridgemarkets(ProductObject $product):void;

    /**
     * Generate an slug, with a given $slug name, normally the product name,
     * if the given name already exists, then starts trying recursively with
     * the $slug.'-'.$count format.
     *
     * @param string $slug
     * @param integer $count
     * @return 
     */
    public function generateSlug(string $name, int $count = 0){

        $name = str_replace(" ","-",strtolower($name));
        $db = new \collection;

        if($count > 0){

            $search = $db->searchForce('OvMdW_posts', ['post_name' => $name.'-'.$count]);
            if($search['cant_rows'] < 1){
                return $name.'-'.$count;
            } else {
                $count++;
                return $this->generateSlug($name, $count);               
            }

        } else {

            $search = $db->searchForce('OvMdW_posts', ['post_name' => $name]);
            if($search['cant_rows'] < 1){
                return $name;
            } else {
                $count++;
                return $this->generateSlug($name, $count);               
            }
            
        }

    }

    /**
     * Checks if the provided $_sku is avaiable for the current user id, if 
     * the product is being updated, then doesn't make the check to the 
     * $post_id provided. Returns true if exist, and false if not.
     *
     * @param integer $user_id
     * @param string $_sku
     * @param integer $post_id
     * @return boolean
     */
    public function is_existing_sku(int $user_id, string $_sku, int $post_id = 0):bool
    {
        $db = new \collection;
        $byProd = ($post_id!=0)?'AND lookup.product_id <> '.$post_id.'':'';
        $results = $db->plainQuery("
        SELECT posts.ID
        FROM 
        ".TABLE_PREFIX."posts as posts
        INNER JOIN ".TABLE_PREFIX."wc_product_meta_lookup AS lookup ON posts.ID = lookup.product_id
        WHERE
        posts.post_type IN ( 'product', 'product_variation' )
        AND posts.post_status != 'trash'
        AND lookup.sku = '".$_sku."'
        AND posts.post_author = ".$user_id."
        ".$byProd."
        LIMIT 1
        ");

        if($results['cant_rows'] < 1){
            return false;
        }

        return true;
    }

    public function maybe_serialize( mixed $data ):mixed
    {
        if ( is_array( $data ) || is_object( $data ) ) {
            return serialize( $data );
        }
        if ( $this->is_serialized( $data, false ) ) {
            return serialize( $data );
        }    
        return $data;
    }

    public function is_serialized(mixed $data, bool $strict = true):bool
    {
        // If it isn't a string, it isn't serialized.
        if (!is_string($data)) {
            return false;
        }
        $data = trim($data);
        if ('N;' === $data) {
            return true;
        }
        if (strlen($data) < 4) {
            return false;
        }
        if (':' !== $data[1]) {
            return false;
        }
        if ($strict) {
            $lastc = substr($data, -1);
            if (';' !== $lastc && '}' !== $lastc) {
                return false;
            }
        } else {
            $semicolon = strpos($data, ';');
            $brace     = strpos($data, '}');
            // Either ; or } must exist.
            if (false === $semicolon && false === $brace) {
                return false;
            }
            // But neither must be in the first X characters.
            if (false !== $semicolon && $semicolon < 3) {
                return false;
            }
            if (false !== $brace && $brace < 4) {
                return false;
            }
        }
        $token = $data[0];
        switch ($token) {
            case 's':
                if ($strict) {
                    if ('"' !== substr($data, -2, 1)) {
                        return false;
                    }
                } elseif (false === strpos($data, '"')) {
                    return false;
                }
                // Or else fall through.
            case 'a':
            case 'O':
                return (bool) preg_match("/^{$token}:[0-9]+:/s", $data);
            case 'b':
            case 'i':
            case 'd':
                $end = $strict ? '$' : '';
                return (bool) preg_match("/^{$token}:[0-9.E+-]+;$end/", $data);
        }
        return false;
    }

}