<?php

namespace BulkUpload\Factories;

use BulkUpload\Factories\Product as ProductFactory;
use BulkUpload\Handlers\Category as CategoryHandler;
use BulkUpload\Handlers\MetaData;
use BulkUpload\Product;
use BulkUpload\Terms\Brand;
use BulkUpload\Terms\Category;
use \BulkUpload\Handlers\Brand as BrandHandler;

class SimpleProduct extends ProductFactory{

    private \collection $db;
    private array $dokanRmaSettings = [
        'label' => 'Garantía',
        'type' => 'no_warranty',
        'policy' => '',
        'reasons' => [],
        'length' => '',
        'length_duration' => '',
        'addon_settings' => []
    ];

    public function __construct(){
        $this->db = new \collection;
    }
    public function insertProduct(Product $product):void{

        $this->insertProductInWordpress($product);
        $this->insertProductInBridgemarkets($product);

    }

    private function generatePostFormatForProduct(Product $product):array{

        $post_format = [];
        if(isset($product->id) && $product->id){
            array_push($post_format, ['ID' => $product->id]);
        }

        $post_format['post_author'] = $product->vendor_id;
        $post_format['post_date'] = $this->currentTime();
        $post_format['post_date_gmt'] = $this->currentTime();
        $post_format['post_content'] = $product->description;
        $post_format['post_title'] = $product->name;
        $post_format['post_excerpt'] = $product->description;
        $post_format['post_status'] = "publish"; //TODO: fix before release
        $post_format['comment_status'] = "closed";
        $post_format['ping_status'] = "closed";
        $post_format['post_password'] = "";
        $post_format['post_name'] = $this->generateSlug($product->name);
        $post_format['to_ping'] = "";
        $post_format['pinged'] = "";
        $post_format['post_modified'] = $this->currentTime();
        $post_format['post_modified_gmt'] = $this->currentTime();
        $post_format['post_content_filtered'] = "";
        $post_format['post_parent'] = 0;
        $post_format['guid'] = "https://staggbm02.bridgemarkets.com/detalle/".$post_format['post_name']; //TODO: Hay que modificar esto
        $post_format['menu_order'] = 0;
        $post_format['post_type'] = 'product';
        $post_format['post_mime_type'] = 'image/jpeg';
        $post_format['comment_count'] = 0;

        //die(print_r($post_format));
        return $post_format;

    }

    private function currentTime() {

        $type = 'Y-m-d H:i:s';
        $timezone = new \DateTimeZone( 'UTC' );
        $datetime = new \DateTime( 'now', $timezone );
    
        return $datetime->format( $type );
    }

    /**
     * Inserts the product using the data in the format to Wordpress Post table.
     */
    public function insertProductInWordpress(Product $product):void{

        if(!$product->id){
            $id = $this->generatePost($product);
            $product->setId($id);
        }
        $this->generatePostMeta($product);
        $this->generateProductMetaLookup($product);
        $this->generatePostBrand($product);
        $this->generatePostCategories($product);
        $this->generatePostTags($product);

    }

    /**
     * Inserts the product in the new product table in BM format.
     *
     * @return void
     */
    public function insertProductInBridgemarkets(Product $product):void{

        $formatedData = [
            'name' => $product->name,
            'slug' => $product->slug,
            'sku' => $product->sku,
            'productType' => $product->productType,
            'price' => $product->price,
            'moq' => $product->moq,
            'description' => $product->description,
            'category' => $product->category,
            'tags' => $product->tags,
            'tariff' => $product->tariff->code,
            'tariffDescription' => $product->tariff->name,
            'model' => $product->model,
            'brand' => $product->brand,
            'country' => $product->country,
            'packageUnit' => $product->packageUnit??"",
            'isDangerousGood' => $product->name??0,
            'vendorAgree' => $product->vendor_aggree,
            'visibility' => $product->_visibility??"visible",
            'stockStatus' => $product->_stock_status??"instock",
            'bulkUploadId' => $product->bulkUploadId??"",
            'unitLength' => $product->unitLength,
            'unitHeight' => $product->unitHeight,
            'unitWeight' => $product->unitWeight,
            'unitWidth' => $product->unitWidth,
            'bulkHeight' => $product->bulkHeight,
            'bulkLength' => $product->bulkLength,
            'bulktWidth' => $product->bulktWidth,
            'bulkWeight' => $product->bulkWeight,
        ];

        $this->db->insert('ng_products', $formatedData);

    }

    private function generatePost(Product $product):int {

        $insertData = $this->generatePostFormatForProduct($product);
        $id = $this->db->insert('OvMdW_posts', $insertData);
        return $id;
    }

    private function generatePostMeta(Product $product):void{

        $postMeta = [
            '_dokan_wholesale_meta' => $this->maybe_serialize([
                "price" => $product->price,
                "quantity" => $product->moq,
                "enable_wholesale" => "yes"
            ]),
            'total_sales' => 0,
            '_tax_status' => 'taxable',
            '_tax_class' => '',
            '_manage_stock' => 'no',
            '_backorders' => 'no',
            '_sold_individually' => '',
            '_virtual' => 'no',
            '_downloadable' => 'no',
            '_download_limit' => -1,
            '_download_expiry' => -1,
            '_stock' => '',
            '_stock_status' => 'instock',
            '_wc_average_rating' => 0,
            '_wc_review_count' => 0,
            '_product_version' => '6.9.4',
            '_regular_price' => $product->price,
            '_price' => $product->price,
            '_visibility' => 'visible',
            '_weight' => $product->unitWeight,
            '_length' => $product->unitLength,
            '_width' => $product->unitWidth,
            '_height' => $product->unitHeight,
            '_custom_product_arancel' => $product->tariff->code,
            '_custom_product_arancel_desc' => $product->tariff->name,
            '_custom_product_model' => $product->model,
            'product_origin_country' => $product->country,
            '_custom_product_brand' => $product->brand,
            '_package_weight' => $product->bulkWeight,
            '_package_length' => $product->bulkLength,
            '_package_width' => $product->bulktWidth,
            '_package_height' => $product->bulkHeight,
            'vendor_aggree' => 'yes',
            '_purchase_note' => '',
            '_product_attributes' => $this->maybe_serialize([]), //TODO: make it compatible with variables
            '_sale_price' => '',
            '_sale_price_dates_from' => '',
            '_sale_price_dates_to' => '',
            '_low_stock_amount' => '',
            '_dokan_rma_override_product' => 'yes',
            '_dokan_rma_settings' => $this->maybe_serialize($this->dokanRmaSettings),
            '_lot_discount_quantity' => 0,
            '_lot_discount_amount' => 0,
            '_is_lot_discount' => 'no',
            '_disable_shipping' => 'no',
            '_overwrite_shipping' => 'no',
            '_additional_price' => '',
            '_additional_qty' => '',
            '_dps_processing_time' => '',
            '_upsell_ids' => $this->maybe_serialize([]),
            '_crosssell_ids' => $this->maybe_serialize([]),
            '_sku' => $product->sku,
            'ekit_post_views_count' => 0,
        ];

        foreach($postMeta as $key => $value){
            $insert = [
                'post_id' => $product->id,
                'meta_key' => $key,
                'meta_value' => $value
            ];
            $this->db->insert('OvMdW_postmeta', $insert);
        }
        
    }
    
    private function generatePostBrand(Product $product):void{
        
        if(Brand::brandExists($product->brand)){
            $brand = Brand::getTermFromName($product->brand);
            $handler = new BrandHandler($brand);
            $handler->saveTermRelationship($product, $brand);
            return;
        }
        $meta_handler = new MetaData();
        $meta_handler->insertMetaData($product->id, "_new_product_brand_name", $product->brand);
        return;

    }

    private function generateProductMetaLookup(Product $product):void{

        $productMeta = [
            'product_id' => $product->id,
            'sku' => $product->sku,
            'virtual' => 0,
            'downloadable' => 0,
            'min_price' => $product->price,
            'max_price' => $product->price,
            'onsale' => 0,
            'stock_status' => 'instock',
            'rating_count' => 0,
            'average_rating' => 0,
            'total_sales' => 0,
            'tax_status' => 'taxable',
            'tax_class' => ''
        ];


        $this->db->insert('OvMdW_wc_product_meta_lookup', $productMeta, $product->id);
        //die("wasaaa");

    }

    private function generatePostCategories(Product $product):void{

        $category = Category::getTermFromId($product->category);
        $manager = new CategoryHandler($category);
        $manager->saveTermAncestors($product, $category);

    }

    private function generatePostTags(Product $product):void{

    }


}