<?php

namespace BulkUpload;

use BulkUpload\Tariff;
use BulkUpload\Country;
use BulkUpload\Error\Error;
use BulkUpload\Error\ProductError;
use BulkUpload\Terms\Category;

class Product
{

    public ?int $id; //
    public string $name; //
    public ?string $slug; //
    public string $sku; // 
    public string $productType;
    public string $price;
    public int $moq;
    public string $description; //
    public int $category;
    public string $tags;
    public Tariff $tariff; //
    public string $model; //
    public string $brand; //
    public string $country; //
    public string $status; // 
    public int $unitLength; //
    public int $unitWeight; //
    public int $unitWidth; //
    public int $unitHeight; //
    public int $bulkLength; //
    public int $bulkWeight;  //
    public int $bulktWidth;  //
    public int $bulkHeight; //
    public ?string $packageUnit = "";
    public ?bool $isDangerousGood = false;
    public ?array $children = [];
    public ?string $vendor_aggree = "yes";
    public string $_visibility; //
    public ?string $_stock_status = "instock";
    public ?string $bulkUploadId = "";
    public int $vendor_id;

    private \collection $db;
    private array $validData = [
        'sku' => '_sku',
        'unitWeight' => '_weight',
        'unitLength' => '_length',
        'unitWidth' => '_width',
        'unitHeight' => '_height',
        'price' => 'price',
        'moq' => 'quantity',
        'status' => 'status',
        'stock_status' => '_stock_status',
        'average_rating' => '_wc_average_rating',
        'rating_count' => '_wc_review_count',
        'tariff' => '_custom_product_arancel',
        'brand' => '_custom_product_brand',
        'model' => '_custom_product_model',
        'country' => 'product_origin_country',
        '_visibility' => '_visibility',
        'bulkWeight' => '_package_weight',
        'bulkLength' => '_package_length',
        'bulktWidth' => '_package_width',
        'bulkHeight' => '_package_height',
        'stock' => '_stock',
        //'attributes' => '_product_attributes',
        'default_attributes' => '_default_attributes',
        'custom_attr_id' => '_custom_attr_id',
        'productType' => 'productType'
    ];

    public function __construct($productData = [])
    {
        $this->db = new \collection;

        if(empty($productData)){
            throw new \Exception("There is no data to initialize a product object", 1);
            ;
        }

        $this->vendor_id = $productData['vendor_id'];
        $this->id = $productData['id']??0;
        $this->name = $productData['name'];
        $this->slug = $productData['slug'];
        $this->sku = $productData['sku'];
        $this->productType = $productData['productType']??'product';
        $this->price = $productData['price'];
        $this->moq = $productData['moq'];
        $this->description = $productData['description'];
        $this->setCategory($productData['category']);
        $this->tags = $productData['tags'];
        $this->setTariff($productData['tariff']);
        $this->model = $productData['model'];
        $this->brand = $productData['brand'];
        $this->setCountry($productData['country']);
        $this->status = $productData['status']??'draft';
        $this->unitLength = $productData['unitLength'];
        $this->unitWeight = $productData['unitWeight'];
        $this->unitWidth = $productData['unitWidth'];
        $this->unitHeight = $productData['unitHeight'];
        $this->bulkLength = $productData['bulkLength'];
        $this->bulkWeight = $productData['bulkWeight'];
        $this->bulktWidth = $productData['bulktWidth'];
        $this->bulkHeight = $productData['bulkHeight'];
        $this->packageUnit = $productData['packageUnit']??'';
        $this->isDangerousGood = $productData['isDangerousGood']??false;
        $this->children = $productData['children']??[];
    }

    public function setId(int $product_id){
        $this->id = $product_id;
    }

    /**
     * Set the value of category
     *
     */
    private function setCategory( string $category):void
    {
        $category = Category::getTermFromName($category);
        $this->category = $category->term_taxonomy_id;
    }

    /**
     * Set the value of tags
     *
     */
    private function setTags($tags)
    {
        $this->tags = $tags;
    }

    /**
     * Set the value of tariff
     *
     */
    private function setTariff($tariff)
    {
        $this->tariff = Tariff::getTariffFromCode($tariff);
    }

    /**
     * Set the value of country
     *
     */
    private function setCountry($country)
    {
        $this->country = $country;
    }

    static public function getProductFromId(int|string $id):Product|Error{

        $mainProductData = (new self)->getMainProductInfoById($id);
        $metaData = (new self)->getMetaData($id);

        //die(var_dump($metaData));

        $productData = array_merge($mainProductData, $metaData);

        die(var_dump($productData));
        return new Product($productData);
        

    }

    static public function getProductFromSku(string $sku, int $vendor_id):Product|Error{

        $product_id = (new self)->getIdFromSKU($sku, $vendor_id);
        return (new Self)->getProductFromId($product_id);

    }

    /*static public function validateProduct(array $productData):Product|ProductError{

        

    }*/

    private function getMainProductInfoById(int|string $id): array{

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_posts.ID,
            ovmdw_posts.post_author,
            ovmdw_posts.post_title,
            ovmdw_posts.post_excerpt,
            ovmdw_posts.post_status,
            ovmdw_posts.post_type,
            ovmdw_posts.post_name,
            ovmdw_posts.post_date,
            ovmdw_posts.post_modified
            FROM
            ovmdw_posts
            WHERE
                ovmdw_posts.post_type IN ('product', 'product_variation')
            AND 
                ovmdw_posts.post_status = 'publish'
            AND 
                ovmdw_posts.ID = '" . $this->db->sanitizeData($id) . "'
            GROUP BY 
            ovmdw_posts.ID
            LIMIT 1"
        );
        $data = $this->db->plainQuery($query);

        if($data['cant_rows'] < 1){
            return new Error('Product', "There is no product registered with the provided ID"); 
        }

        $productData = [
            'id' => $data[0]['ID'],
            'name' => $data[0]['post_title'],
            'slug' => $data[0]['post_name'],
            'description' => $data[0]['post_excerpt'],
            'status' => $data[0]['post_status'],
            'status' => $data[0]['post_type']
        ];

        return $productData;

    }

    private function getMetaData(int|string $id):array{

        $productMetaData = [];

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_key,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.post_id = ".$id."
            AND
            ovmdw_postmeta.meta_key IN ('" . implode("','", $this->validData) . "', '_dokan_wholesale_meta')
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        foreach ($data as $key => $value) {
            if($key == 'cant_rows'){
                continue;
            }
            //die(var_dump($value));
            $productMetaData = array_merge($productMetaData, $this->manageMeta($value));
            //die(var_dump($this->manageMeta($value)));
            
        }

        return $productMetaData;


    }

    private function manageMeta(array $metaData):array{

        $returnedValues = [];

        switch ($metaData['meta_key']) {
            case '_dokan_wholesale_meta':
                $metaObject = new \MetaValues();
                $dt =  $metaObject->maybe_unserialize($metaData['meta_value']);

                $index = array_search('price', $this->validData);
                $returnedValues[$index] = $metaObject->maybe_serialize($dt['price']);
                
                $index = array_search('quantity', $this->validData);
                $returnedValues[$index] = $metaObject->maybe_serialize($dt['quantity']);
                break;
            
            default:
                $index = array_search($metaData['meta_key'], $this->validData);
                $returnedValues[$index] = $metaData['meta_value']; 
                break;

            //TODO: I need to develop the cases for variable products
        }

        return $returnedValues;

    }

    public function getIdFromSKU(string $sku, string|int $vendor_id):int|Error{

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_key,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.meta_key = '_sku' AND
            ovmdw_postmeta.meta_value = '".$sku."'
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        die($query);
        $skus = $this->db->plainQuery($query);

        if($skus['cant_rows'] < 1){
            return new Error('Product', 'There is no product with the given sku');
        }

        $ids = [];
        foreach ($skus as $key => $sku) {
            if($key == 'cant_rows'){
                continue;
            }
            array_push($ids, $sku['ID']);
        }

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_posts.ID,
            OvMdW_posts.post_author
            FROM
            OvMdW_posts
            WHERE
            OvMdW_posts.ID IN (".implode(",", $ids).")
            AND
            OvMdW_posts.post_author = ".$vendor_id."
            GROUP BY
            OvMdW_posts.ID                    
            "
        );

        $data = $this->db->plainQuery($query);
        if($data['cant_rows'] < 1){
            return new Error('Product', 'There is no product with the given vendor id and sku combination');
        }

        return $data[0]['ID'];

    }

    private function getCategories(){

    }

    private function getTags(){

    }

}
