<?php

namespace  InventoryManagement;

class InventoryCron
{

    use InventoryCronBM, InventoryCronVendor;

    private array $vendors = [];
    private $wp_api_root;
    private \collection $db;
    private $log = [
        'total_vendors' => 0,
        'total_email_sending_failures' => 0,
        'new_processes_started' => 0,
    ];
    public $cronDate;

    public function __construct(string $wp_api_root)
    {
        $this->db = new \collection;
        $this->wp_api_root = $wp_api_root;
        $this->setVendorList();
    }

    /**
     * Sets the vendor list for the usage on the class. The $wp_api_root parameter is used in other child method.
     *
     * @param string $wp_api_root
     * @return void
     */
    private function setVendorList()
    {

        $vendor_query = str_replace(
            'OvMdW_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_users.ID,
            OvMdW_users.user_email,
            OvMdW_usermeta.meta_value AS store_name,
            Count(OvMdW_posts.ID) AS product_count
            FROM
            OvMdW_posts
            INNER JOIN OvMdW_users ON OvMdW_users.ID = OvMdW_posts.post_author
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'dokan_store_name' AND
            OvMdW_posts.post_status = 'publish' AND
            OvMdW_posts.post_type = 'product'   
            GROUP BY
            OvMdW_usermeta.meta_value,
            OvMdW_users.user_email,
            OvMdW_users.ID
            HAVING
            store_name <> ''"
        );
        $vendor_results = $this->db->plainQuery($vendor_query);

        if ($vendor_results['cant_rows'] > 0) {
            foreach ($vendor_results as $key => $val) {

                if ($key == 'cant_rows') {
                    continue;
                }

                $this->vendors[$val['ID']] = array(
                    'ID' => $val['ID'],
                    'email' => $val['user_email'],
                    'store_name' => $val['store_name'],
                    'product_count' => $val['product_count']
                );
            }
        } else {
            throw new \Exception("No hay vendedores disponibles con productos publicados", 1);
            return;
        }

        $this->log['total_vendors'] = count($this->vendors);

        $this->checkAvailability();
    }

    public function initCron():array
    {
        return $this->log;
    }

    private function checkInventoryStatus($vendorTrackingData)
    {

        if (!$vendorTrackingData) {
            return "not_called";
        }

        switch ($vendorTrackingData['still_working']) {
            case 0:
                return "inactive";
                break;
            case 1:
                return "active";
                break;
        }
    }

    private function getVendorTrackingData($id_user)
    {
        $res = $this->db->plainQuery("SELECT * FROM `ng_inventory_tracker` WHERE `ng_inventory_tracker`.`id_user`='" . $this->db->sanitizeData($id_user) . "'
        ORDER BY id DESC LIMIT 1", force:true);

        if ($res['cant_rows'] < 1) {
            return false;
        }

        return $res[0];
    }

    private function generateInventoryToken($vendorData)
    {
        $identifierHash = wp_generate_uuid4();

        $mailSendStatus = 1;
        if(!$this->inventoryManageLister($this->wp_api_root, $vendorData['store_name'], $vendorData['email'], $identifierHash)){
            $mailSendStatus = 0;
            $this->log['total_email_sending_failures']++;
        }
        
        $data = [
            'id_user' => $vendorData['ID'],
            'name' => $vendorData['store_name'],
            'login_user' => $vendorData['user_email'],
            'identifier' => $identifierHash,
            'last_call' => date('Y-m-d'),
            'still_working' => 1,
            'email_sended' => $mailSendStatus
        ];

        $this->db->insert('ng_inventory_tracker', $data);
        $this->log['new_processes_started']++;

    }

    private function getVendorProducts($id_user){
        $products = $this->db->plainQuery("SELECT
            *
            FROM
            OvMdW_posts
            WHERE
            OvMdW_posts.post_type = 'product' AND
            OvMdW_posts.post_status = 'publish' AND
            OvMdW_posts.post_author = ".$this->db->sanitizeData($id_user));

        $products_id = [];

        if($products['cant_rows']>0){

            foreach ($products as $val) {

                if(!isset($val['ID'])){
                    continue;
                }
                array_push($products_id, $val['ID']);

            }

        }
        //die(print_r($products_id));

        return $products_id;
    }

    private function listingEmailHasBeenSent($identifier){

        $this->db->update('ng_inventory_tracker', ['identifier'=>$identifier], ['email_sended'=>'1']);

    }

    private function getDistanceDays(\DateTime $lastCallDate)
    {
        $this->cronDate = $lastCallDate;
        $currentDate = new \DateTime("now");
        return $lastCallDate->diff($currentDate)->d;
    }

    private function inactiveLinkActions(array $vendorTrackingData, array $vendorData): void
    {

        $daysPassed = $this->getDistanceDays(new \DateTime($vendorTrackingData['last_call']));
        if ($daysPassed == 15) {
            $this->generateInventoryToken($vendorData);
        }
    }

    private function activeLinkActions(array $vendorTrackingData, array $vendorGeneralData): void
    {
        $lastCall = new \DateTime($vendorTrackingData['last_call']);
        $daysPassed = $this->getDistanceDays($lastCall);

        if($vendorTrackingData['email_sended'] == 0){

            if(!$this->inventoryManageLister($this->wp_api_root, $vendorGeneralData['store_name'], $vendorGeneralData['email'], $vendorTrackingData['identifier'], 15 - $daysPassed)){
                $this->log['total_email_sending_failures']++;
            } else {
                $this->listingEmailHasBeenSent($vendorTrackingData['identifier']);
            }
        }
        if ($daysPassed == 15) {
            $this->inventoryWasNotUpdated($daysPassed, $vendorGeneralData['store_name'], $vendorGeneralData['product_count']);
            $this->productsToDraftNotification($daysPassed, $vendorGeneralData['store_name'], $vendorGeneralData['email'], $vendorGeneralData['product_count']);
            $handleStates = new ProductManager;
            $productIds = $this->getVendorProducts($vendorGeneralData['ID']);
            $handleStates->handleProductState($vendorTrackingData['identifier'], inactiveProducts:$productIds, fast:true);
            if($this->checkIfNewProductsWereCreated($vendorGeneralData['ID'])){
                $this->db->update('ng_inventory_tracker', ['identifier'=>$vendorTrackingData['identifier']], ['still_working'=>'0']);
                $this->generateInventoryToken($vendorGeneralData);
            }
        } else if ($daysPassed == 5 || $daysPassed == 10) {
            $this->inventoryManageReminder(intval($daysPassed), $vendorGeneralData['store_name'], $vendorGeneralData['email'], $vendorTrackingData['identifier']);
        }
    }

    public function checkIfNewProductsWereCreated(string $vendor_id): bool
    {
        $res = $this->db->plainQuery("SELECT * FROM OvMdW_posts WHERE post_type='product' AND post_status='publish' AND post_author='".$vendor_id."' AND post_date >='".$this->cronDate->format('Y-m-d H:i:s')."'", force:true);
        if($res['cant_rows'] > 0){
            return true;
        }
        return false;
    }

    public function checkAvailability()
    {

        foreach ($this->vendors as $val) {

            if ($val['ID'] != '804' && $val['ID'] != '49' && $val['ID'] != '808' && $val['ID'] != '815') {
                continue;
            }

            $vendorTrackingData = $this->getVendorTrackingData($val['ID']);

            $status = $this->checkInventoryStatus($vendorTrackingData);
            switch ($status) {

                case 'not_called':
                    $this->generateInventoryToken($val);
                    break;

                case 'inactive':
                    $this->inactiveLinkActions($vendorTrackingData, $val);
                    break;

                case 'active':
                    $this->activeLinkActions($vendorTrackingData, $val);
                    break;
            }
        }
    }
}
