<?php

namespace  InventoryManagement;

trait InventoryCronBM {

    /**
     * Sends an email in the case that the inventory was updated. Returns true if the emails was sent succesfully.
     *
     * @param string $vendorName
     * @param array $unavailableProducts
     * @return void
     */
    public function inventoryIsUpdated(string $identifier, array $availableProducts = []){

        $db = new \collection;
        $mailer = new \email;

        $res = $db->search('ng_inventory_tracker', ['still_working ' => '1', 'identifier' => $identifier], 'AND', 'DEFAULT_WHERE', '1');

        if($res['cant_rows'] < 1){
            throw new \Exception("No ha proveido un identificador valido.", 1);
            return;
        }

        $vendor_query = str_replace(
            'OvMdW_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_users.ID,
            OvMdW_users.user_email,
            OvMdW_usermeta.meta_value AS store_name,
            Count(OvMdW_posts.ID) AS product_count
        FROM
            OvMdW_posts
        INNER JOIN OvMdW_users ON OvMdW_users.ID = OvMdW_posts.post_author
        INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
        WHERE
            OvMdW_usermeta.meta_key = 'dokan_store_name'
        AND OvMdW_posts.post_status = 'publish'
        AND OvMdW_posts.post_type = 'product'
        AND OvMdW_users.ID = ".$db->sanitizeData($res[0]['id_user'])."
        GROUP BY
            OvMdW_usermeta.meta_value,
            OvMdW_users.user_email,
            OvMdW_users.ID
        HAVING
            store_name <> ''"
        );
        $vendor_results = $this->db->plainQuery($vendor_query);

        if($vendor_results['cant_rows'] < 1){
            throw new \Exception("No hay informacion acerca de ese vendedor.", 1);
            return;
        }

        $availableProductsNew = $this->db->plainQuery("SELECT
        OvMdW_posts.ID,
        OvMdW_posts.post_author,
        OvMdW_posts.post_title,
        OvMdW_posts.post_excerpt,
        OvMdW_posts.post_status,
        OvMdW_posts.post_type,
        OvMdW_posts.post_name,
        OvMdW_posts.post_date
    FROM
        OvMdW_posts
    WHERE
        OvMdW_posts.post_type = 'product'
    AND OvMdW_posts.post_status = 'publish'
    AND OvMdW_posts.post_author = '804'
    GROUP BY
        OvMdW_posts.ID
    HAVING
        OvMdW_posts.post_status = 'publish'
    AND OvMdW_posts.post_type = 'product'
    AND OvMdW_posts.ID NOT IN (
        SELECT
            OvMdW_postmeta.post_id
        FROM
            OvMdW_postmeta
        WHERE
            OvMdW_postmeta.meta_key = '_visibility'
        AND OvMdW_postmeta.meta_value = 'hidden'
        GROUP BY
            OvMdW_postmeta.post_id
    )
    AND
        OvMdW_posts.ID NOT IN (".implode(",", $availableProducts).")
    ORDER BY
        OvMdW_posts.post_title ASC,
        OvMdW_posts.post_excerpt ASC ");

        $unavailableProductsList = '';

        foreach ($availableProductsNew as $product) {
            if($product != ''){
                $unavailableProductsList .= '<tr><td><img src="'.$product['images'][0]['url'].'" alt=""></td> <td><a href="'.$product['url'].'">'.$product['name'].'</a></td> <td>'.$product['sku'].'</td> <td><input class="form-check-input" type="checkbox" name="approve_products[]" class="lines" value="'.$product['product_id'].'"></td> </tr>';
            }
        }

        $content = "El vendedor ".$vendor_results['store_name']." ha hecho uso de la herramienta de certificación de inventario y afirma NO tener disponibilidad de los siguientes productos: <br><br><ul>".$unavailableProductsList."</ul>";

        $mailer->set_multidestination(OPERATIONS_EMAIL_NOTIFICATIONS);
        $mailer->set_template('Revisión de productos disponibles', $content);

        if (!$mailer->send_mail()){
            return false;
        }

        return true;

    }

    /**
     * Sends an email in the case that after the 15 days, the vendor didn't confirm the stock of the published products.Returns true if the emails was sent succesfully.
     *
     * @param integer $daysPassed
     * @param string $vendorName
     * @param integer $productCount
     * @param integer $daysToValidate
     * @return boolean
     */
    public function inventoryWasNotUpdated(int $daysPassed, string $vendorName, int $productCount, int $daysToValidate = 15):bool{

        if($vendorName == ''){
            throw new \Exception("No fue proporcionado ningún nombre de vendedor.", 1);
        }

        if($productCount < 1){
            throw new \Exception("No hay productos que revisar.", 1);
        }

        if($daysPassed < $daysToValidate){

            return false;

        }

        $content = "El vendedor ".$vendorName." no actualizó su inventario, por lo que de manera preventiva sus ".$productCount." productos publicados pasaron automáticamente a estado Borrador.";

        $mailer = new \email;
        $mailer->set_multidestination(OPERATIONS_EMAIL_NOTIFICATIONS);
        $mailer->set_template($vendorName.' - Productos no actualizados', $content);

        if (!$mailer->send_mail()){
            return false;
        }

        return true;

    }

    /**
     * Sends a custom notificaction via email related to the inventory. Returns true if the emails was sent succesfully.
     *
     * @param string $title
     * @param string $content
     * @param array $destination
     * @return boolean
     */
    public function inventoryCustomNotification(string $title, string $content, array $destination):bool{

        if($title == ''){
            throw new \Exception("No fue proporcionado ningún título.", 1);
        }

        if($content == ''){
            throw new \Exception("No fue proporcionado ningún contenido.", 1);
        }

        if(count($destination) < 1){
            throw new \Exception("No fue proporcionado ningún destinatario.", 1);
        }
        
        $mailer = new \email;
        $mailer->set_multidestination($destination);
        $mailer->set_template($title, $content);

        if (!$mailer->send_mail()){
            return false;
        } 
        
        return true;

    }


}