<?php 

class Dbs_Class implements Currier
{
    private $db;
    private  Package  $package;
    private Destination $destination;
    private $tax = 0;
    private $dai = 0;
    private $total=0;
    private $manejo = 15;
    private array $arancelData = [];
    private  $zoneShippingPrice = [
        'interior'=>['minimumWeightPricing' => 90, 'aditionalWeight' =>3.5, 'logisticPrice'=>30],
        'ciudad'=>['minimumWeightPricing' => 30, 'aditionalWeight' =>2.5, 'logisticPrice'=>25]
    ]; 
    private $MINIMUMWEIGHT = 20;
    function __construct()
    {
        $this->db = new collection;
    }
  

    private function calculateWeightPrice(Package $package, Destination $directions){
        $zoneOfShipping = strcasecmp($directions->city,'Panama') == 0?'ciudad':'interior'; 
        if($package->getTotalCBM()['air']<0){
            throw new ErrorException('invalid CBM value');
        }
        $this->total += $this->zoneShippingPrice[$zoneOfShipping]['minimumWeightPricing'] +$this->zoneShippingPrice[$zoneOfShipping]['logisticPrice'] +$this->manejo ;
        if($package->getTotalCBM()['air']>$this->MINIMUMWEIGHT){
            $this->total += $this->zoneShippingPrice[$zoneOfShipping]['aditionalWeight']*($package->getTotalCBM()['air']-20);
            
        }
        //die(var_dump($this->total));

    }
    private function calculateArancelTax( $item, $dai)
    {
       // die(var_dump($item->get_product_arancel()));
        $value = 0;
        if (isset($this->arancelData[$item->get_product_arancel()])) {
            $value = ($this->arancelData[$item->get_product_arancel()]['itbms'] / 100) * (($item->getTotalAmount()));
        }
        $this->total += $value;
        
    }
    public function calculateDai(PackageItem $item)
    {
        $value = 0;
        if (isset($this->aracelData[$item->get_product_arancel()])) {
            $value = ($this->arancelData[$item->get_product_arancel()]['dai'] / 100) * $item->getTotalAmount();
            $this->dai += $value;
        }
        return 0;
    }
    private function calculateShippingTax(){
        $shippingValue = $this->total;
        $this->tax += $shippingValue*0.22;
            
    }
    private function loadArancelDetails(): void
    {
        $arancelList = [];
        foreach ($this->package->getItems() as $item) {
            if ($item instanceof PackageItem) {
                array_push($arancelList, $item->get_product_arancel());
            }
        }
        $data = $this->db->plainQuery("SELECT *  From ovmdw_dmce_hs_codes_complete WHERE dmce_hs_codes IN (" . implode(',', $arancelList) . ")");
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->arancelData[$data[$i]['dmce_hs_codes']] = $data[$i];
        }
    }

    public function generateResponseSchema(array $data, bool $include_taxes = false)
    {
        $res = [];

        $res['method_title'] = 'DBS';
        $res['description'] = 'DBS Logistic';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['message'] = 'Acceso a la Entrega:

Es responsabilidad del cliente garantizar acceso adecuado al lugar de entrega. De lo contrario, podrían aplicarse cargos adicionales.';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/2024/09/568b29e6-5e76-4b2c-bb6d-f38632115fd1.jpg';
        $prod['type'] = 'ground';
        $prod['productName'] = 'Tiempos de Entrega Estimados:

El tiempo de entrega es de 24 a 48 horas hábiles en la ciudad de Panamá y 48 a 72 horas hábiles en el interior del país.

Los envíos dentro de Panamá se realizan únicamente al área metropolitana.

';
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = round($this->total, 2);
        $prod['volumetric'] = $this->package->getTotalCBM()['air'];
        $prod['deliveryCapabilities']['totalTransitDays'] = 0;
        $prod['details'][] = $this->insertDetail('Envío', '', round($this->total,2));
        $res['options'][] = $prod;
        //die(var_dump($res));
        return $res;
    }
    function errorResponse(){
        return;
    }
    function checkout_rate(Destination $_item, Package $package, bool $include_taxes){
        $arancelTax = 0;
        $this->destination = $_item;
        $this->package = $package;
        $this->loadArancelDetails();
        $this->calculateWeightPrice($this->package,$this->destination);
        foreach ($this->package->getItems() as $item) {
            if ($item instanceof PackageItem) {
                $dai = $this->calculateDai($item);
                $this->calculateArancelTax($item, $dai);
            }
        }
        $this->calculateShippingTax();
        $this->total += $this->tax;

        //die(var_dump($this));
        if ($this->total != 0) {
            return $this->generateResponseSchema([], $include_taxes);
        }
    }
    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }
}