<?php

class FEDEX_class implements Currier
{
    private $url = FEDEX_URL;
    private $apiKey = FEDEX_ACCESS_KEY;
    private $apiSecret = FEDEX_ACCESS_SECRET;
    private $accountType = FEDEX_ACCOUNT_TYPE;
    private $accountNumber = FEDEX_ACCOUNT_NUMBER;
    private $serviceType = 'FEDEX_INTERNATIONAL_CONNECT_PLUS';
    private $pickupType = 'CONTACT_FEDEX_TO_SCHEDULE';
    private $packingType = 'YOUR_PACKAGING';
    private $jwt = '';
    private $productCode = 'P';
    private $isCustomsDeclarable = true;
    private $getRateEstimates = true;
    private $pickupDelay = '+1 day';

    private $currentDate;
    private $pickupDate;
    private $currentDateString;
    private $pickupDateString;
    private $request = '';
    public $error = '';
    public $response = [];
    private $request_id;
    public $packingList = [];
    private $senderData = [];
    private $customerData = [];
    private $packageData = [];
    private Package $package;
    private $_tracking_document = '';
    private $_shipmentTrackingNumber = '';
    private $_shipmentPickUpDate = '';
    public $status = true;
    private $errorcodes = [
        'COUNTRY.POSTALCODEORZIP.INVALID' => 'El código postal no es válido para el país seleccionado. Corríjalo e intente nuevamente.',
        'CURRENCY.TYPE.INVALID' => 'El tipo de moneda que seleccionó no es válido. Seleccione otro tipo de moneda.',
        'CURRENCYTYPE.CADORUSD.REQUIRED' => 'CAD o US',
        'CUSTOMVALUE.LIMITEXCEEDS.ERROR' => 'El valor total para la aduana declarado ingresado supera el límite de los servicios internacionales de FedEx®. Para obtener información adicional, así como otros servicios disponibles, comuníquese con el Servicio al Cliente de FedEx.',
        'DIMENSIONS.EXCEEDS.LIMITS' => 'Las dimensiones superan la dimensión máxima para este servicio. Consulte la Guía de Servicios de FedEx.',
        'POSTALCODE.ZIPCODE.REQUIRED' => 'El código postal es obligatorio',
        'CURRENCY.TYPE.MISMATCH' => 'El tipo de moneda de valor declarado debe ser igual al tipo de moneda de valor en aduana.',
        'ACCOUNT.NUMBER.INVALID' => 'Número de cuenta no válido.',
        'ACCOUNTNUMBER.ENROLLED.INVALID' => 'El número de cuenta del envío no está registrado en el servicio FedEx Priority Alert(TM) .',
        'ACCOUNTNUMBER.ONERATE.NOTAVBL' => 'La Tarifa Única de FedEx® no está disponible para esta cuenta. Utilice la Tarifa Estándar de FedEx.',
        'CARRIAGE.VALUE.EXCEEDSMAXIMUM' => 'El valor declarado para transporte ingresado excede el máximo. Verifíquelo e ingréselo nuevamente',
        'CARRIAGEVALUE.CUSTOMVALUE.VALIDATION' => 'Las tarifas y los tiempos de tránsito no están disponibles para el número de cuenta ingresado. Comuníquese con el Servicio al Cliente de FedEx local para recibir asistencia inmediata.',
        'CITY.REQUIRED' => 'Debe brindar una ciudad válida.',
        'COUNTRY.POSTALCODE.INVALID' => 'El código postal de origen es obligatorio o no es válido.',
        'SHIPPER.POSTALSTATE.MISMATCH' => 'El código postal y el estado del remitente no coinciden. Corrija la información e inténtelo de nuevo.',
        'CURRENCY.MISSMATCH.ERROR' => 'Paquete: la moneda del valor asegurado {CURRENCY_TYPE_1} no coincide con la moneda del valor para aduana {CURRENCY_TYPE_2}.',
        'CURRENCY.TYPE.SELECT' => 'Ingrese el tipo de moneda',
        'CURRENCYAMOUNT.CURRENCYTYPE.INVALID' => 'El valor máximo declarado es de {CURRENCY_AMOUNT} {CURRENCY_TYPE}. Comuníquese con el Servicio al cliente de FedEx para obtener más información.',
        'CURRENCYTYPE.CADORUSD.ONLYAVAILABLEERROR' => 'De acuerdo con la información ingresada, este envío califica para FedEx Home Delivery®. Vuelva a enviar su solicitud como FedEx Home Delivery para continuar con el envío.',
        'DELIVERY.DAY.ERROR' => 'Elija Saturday Pickup (Recolección el sábado) o Saturday Delivery (Entrega el sábado).',
        'DIMENSION.EXCEEDS.MAXMUM' => 'Las dimensiones ingresadas superan la dimensión máxima para este servicio. Consulte la Guía de servicios de FedEx.',
        'DIMENSION.TYPE.ERROR' => 'Las dimensiones que ingresó exceden el límite para el origen y el destino seleccionados; verifíquelas e ingréselas nuevamente.',
        'DIMENSIONS.EXCEEDS.LIMIT' => 'Las dimensiones que ingresó exceden el límite de altura permitido; verifíquelas e ingréselas nuevamente.',
        'ENTERED.ZIPCODE.NOTFOUND' => 'No se encontró el estado o la provincia ni el código postal ingresado, o el estado o la provincia y el código postal no cuentan con el servicio de FedEx en este país.',
        'SERVICE.PACKAGECOMBINATION.INVALID' => 'Combinación de servicio y embalaje no válida.',
        'FEDEXHOMEDELIVERY.SERVICE.QUALIFICATIONERROR' => 'De acuerdo con la información ingresada, este envío califica para FedEx Home Delivery®. Vuelva a enviar su solicitud como FedEx Home Delivery para continuar con el envío.',
        'FEDEXSERVICE.NOT.AVAILABLE' => 'FedEx no ofrece servicios hacia el país de destino o desde el país de origen en este momento. Comuníquese con el Servicio al Cliente de FedEx.',
        'FREIGHTGUARANTEE.TIMEVALUE.REQUIRED' => 'Se requiere un valor temporal válido de garantía de cargamento.',
        'FREIGHTGUARANTEEE.TIME.REQUIRED' => '{PACKAGING_TYPE} no puede superar {weight}{weight_units}. Seleccione otro tipo de embalaje.',
        'DECLAREDVALUE.NOT.ALLOWED.FOR.SMARTPOST' => 'No se permite el valor declarado para SmartPost.',
        'LOCATION.PICKUP.NOTAVAILABLE' => 'Su ubicación no admite recolección. Seleccione Dejar en oficina de FedEx para dejar su paquete en una oficina de FedEx o comuníquese con el Servicio al Cliente de FedEx.',
        'MAXIMUMWEIGHT.TYPE.ERROR' => 'El peso máximo permitido para esta aplicación es de 68 kg/150 lb. Reingrese el peso o comuníquese con el Servicio al Cliente de FedEx.',
        'ORIGIN.COUNTRY.REQUIRED' => 'Debe brindar un país de origen válido.',
        'ORIGIN.PICKUP.ERROR' => 'El origen que seleccionó no permite la recolección para los servicios FedEx Express Freight. Puede comunicarse con el Servicio al Cliente de FedEx local (en los Estados Unidos y Canadá, llame al 1.800.GoFedEx [1.800.463.3339]) para obtener información sobre otras opciones de recolección de envíos.',
        'ORIGIN.PICKUP.NOTALLOWED' => 'El valor declarado para transporte ingresado excede el máximo. Verifíquela e ingrésela nuevamente.',
        'PACKAGE.INVALID.TYPE' => 'Sección II No se permiten baterías/celdas de litio con un envío de documentos.',
        'PACKAGE.WEIGHT.INVALID' => 'Falta el peso del paquete o no es válido.',
        'PACKAGEDIMENSION.TYPE.ERROR' => 'El paquete debe tener, por lo menos, 1 cm/pulgada de longitud, de ancho y de altura. Si no está seguro de las dimensiones, puede dejar estos campos en blanco.',
        'RATE.SPECIALSERVICETYPE.NOTALLOWED' => 'El Servicio especial {0} no está permitido.',
        'PICKUPTYPE.NOT.AVAILABLE' => 'La recolección no está disponible. Cambie el tipo de servicio, la fecha de envío o seleccione entrega para completar el envío.',
        'RATING.CARRIAGE.EXCEEDSLIMITS' => 'El valor declarado para transporte excede el límite permitido según la información ingresada.',
        'RATING.CARRIAGLIMITS.EXCEED' => 'Las dimensiones superan la dimensión máxima para este servicio. Consulte la Guía de Servicios de FedEx.',
        'RATING.DIMENSIONSEXCEEDS.LIMITS' => 'El valor para la aduana supera el límite de {CURRENCY_AMOUNT} {CURRENCY_TYPE}.',
        'RATING.DIMENSIONSTYPE.EXCEEDSLIMITS' => 'El envío de mercancías peligrosas no ha sido habilitado para su cuenta. Comuníquese con la línea local de Servicio al cliente de FedEx para recibir asistencia.',
        'RATING.ZIPCODE.INVALID' => 'Información sobre tarifa y tiempo de envío no disponible. Intente nuevamente más tarde o comuníquese con el Servicio al Cliente de FedEx',
        'RATE.PACKAGES.WEIGHT.UNIT.MISMATCH' => 'La unidad de medida de peso debe ser igual para todos los paquetes.',
        'RESIDENTIAL.DELIVERY.NOTAVAILABLE' => 'Residential delivery (Entrega residencial) no está disponible para el servicio seleccionado.',
        'SERVICE.LOCATION.UNAVAILABLE' => 'El servicio dentro del país no está disponible en este momento./El servicio de FedEx no está disponible actualmente para esta combinación de origen (o dos mensajes) y destino. Puede ingresar información nueva o comunicarse con el Servicio al Cliente de FedEx local (en los Estados Unidos y Canadá, llame al 1.800.GoFedEx [1.800.463.3339]).',
        'SERVICETYPE.FORPOSTALCODE.NOTALLOWED' => 'El valor declarado para transporte ingresado excede el máximo. Verifíquela e ingrésela nuevamente.',
        'SHIPMENT.DGDIVIOLATIONS.NOTALLOWED' => 'Su envío no está permitido debido a violaciones de mercancías peligrosas o hielo seco que limitan sus opciones de envío. Si tiene alguna pregunta, comuníquese con la línea local de Servicio al cliente de FedEx.',
        'SHIPMENT.DGVIOLATIONS.NOTALLOWED' => 'Su envío no está permitido debido a violaciones de mercancías peligrosas que limitan sus opciones de envío. Si tiene alguna pregunta, comuníquese al 1-800-463-339 y presione “81” para acceder a la línea directa de mercancías peligrosas / materiales peligrosos de FedEx.',
        'WEIGHT.BELOWMINIMUMLIMIT.ERROR' => 'El peso ingresado es inferior al mínimo estándar para los servicios de FedEx Express {package}. Ingrese un peso de por lo menos {WEIGHT} {WEIGHT_UNITS}.',
        'SHIPMENTPROCESSING.ACCOUNTVIOLATIONS.ERROR' => 'No hay servicios devueltos debido a violaciones de mercancías peligrosas, materiales peligrosos o hielo seco en su cuenta.',
        'STATE.TYPE.INVALID' => 'Debe brindar un(a) estado/provincia válido(a).',
        'STATE.TYPE.REQUIRED' => 'El estado o la provincia es obligatorio',
        'SUBTYPE.SELECT.FORBATTERIES' => 'Sección II Debe seleccionarse Baterías/Celdas de Litio y uno de los subtipos.',
        'WEIGHT.TYPE.INVALID' => 'Ingrese un peso válido.',
        'RATE.QUOTENO.NOTFOUND' => 'No se pudo encontrar el número de cotización de tarifa',
        'ACCOUNTNBR.MISSING.ORINVALID' => 'El valor del tipo de condición ACCOUNT_NUMBER está en blanco o no es válido.',
        'SPECIAL.SERVICE.CONFLICT' => 'Conflicto de servicio especial. Retener en oficina no es válido con una entrega residencial.',
        'DRYICEWEIGHT.REQUIRED.INKGS' => 'Paquete {0}: se requiere el peso del hielo seco (en kilogramos) para el servicio especial Hielo seco.',
        'SERVICETYPE.PICKUPMETHOD.NOT.ALLOWED' => '{0} no está permitido con el método de recolección.',
        'FXSPRETURN.RATES.ERROR' => 'Las tarifas para devolver envíos de SmartPost no están disponibles en este momento.',
        'RATE.SPECIALSERVICE.NOTALLOWED.ORIGIN' => '{0} no está permitido para el origen.',
        'PACKAGINGTYPE.MISSING.OR.INVALID' => 'El tipo de embalaje no fue indicado o no es válido.',
        'RATE.SPECIALSERVICE.NOT.ALLOWED' => 'Paquete {0}: no se permite el servicio especial {1}.',
        'SMARTPOST.HUBID.INVALID' => 'La ID del centro de SmartPost no es válida.',
        'SPECIALSERVICE.NOT.ALLOWED.SHIPMENTORPACKAGE' => 'El servicio especial {0} no está permitido en el nivel {1}.',
        'SPECIALSERVICE.NOT.SUPPORTED.REQUESTED.SHIPDAY' => '{0} no tiene soporte para el día de la semana del envío requerido. Actualice la información e inténtelo nuevamente.',
        'CODDETAIL.EMPTY.ORNULL' => 'Paquete {0}: CodDetail no puede ser nulo.',
        'DESTINATION.POSTALCODE.MISSING.ORINVALID' => 'Falta el código postal de destino o no es válido.',
        'TOTALCUSTOMSVALUE.CURRENCYTYPE.INVALID' => 'Tipo de moneda no válido para el valor total de aduana.',
        'RATE.PACKAGES.DIMENSION.UNIT.MISMATCH' => 'La unidad de medida de la dimensión debe ser igual para todos los paquetes.',
        'COMMODITY.NUMBEROFPIECES.REQUIRED' => 'Mercancía {COMMODITY_INDEX}: se requiere el número de piezas de mercancías y el valor no puede ser negativo o superior a 9999.',
        'PICKUPTYPE.REQUIRED' => 'El elemento PickupType es obligatorio en la solicitud de entrada.',
        'ORIGIN.COUNTRY.INVALID' => 'El código del país de origen falta o no es válido. Consulte en la documentación el formato válido.',
        'DESTINATION.COUNTRY.INVALID' => 'El código del país de destino falta o no es válido. Consulte en la documentación el formato válido.',
        'MAXIMUM.WEIGHT.EXCEEDED' => 'No se admite el peso del paquete en la solicitud. El peso máximo del paquete para todos los servicios es de 2 200 lb o 998 kg.',
        'ACCOUNT.NUMBER.MISMATCH' => 'Cuando el tipo de pago es REMITENTE, el número de cuenta del pagador de los cargos de envío debe coincidir con el número de cuenta del remitente.',
        'SHIPDATESTAMP.FORMAT.INVALID' => 'Proporcione un formato válido shipDatestamp AAAA-MM-DD',
        'INTERNAL.SERVER.ERROR' => 'Se produjo un error inesperado y estamos trabajando para resolver el problema. Lamentamos las molestias. Vuelva a intentarlo más tarde.',
        'RATING.DESTINATIONSTATEPOSTAL.MISMATCH' => 'El código postal del estado de destino no coincide.',
        'ACCOUNT.NUMBERORKEY.INVALID' => 'Información de cuenta inválida. El número de cuenta o la clave son inválidos.',
        'RATING.QUOTE.NOTAVAILABLE' => 'Las tarifas y los tiempos de tránsito no están disponibles para el número de cuenta ingresado. Comuníquese con el Servicio al Cliente de FedEx local para recibir asistencia inmediata.',
        'HIPPER.COUNTRY.INVALID' => 'El código de país del remitente falta o no es válido. Actualice la información e inténtelo de nuevo.',
        'FREIGHTSERVICEORIGIN.FREIGHT.NOTALLOWED' => 'El origen no cuenta con servicios de FedEx Freight.',
        'FREIGHTSERVICEORIGIN.PICKUP.NOTALLOWED' => 'El origen no permite la recolección para servicios de FedEx Freight.',
        'REGIONAL.MAILPICKUP.NOTALLOWED' => 'No se permite la opción de recolección en el correo regional.',
        'REGIONAL.MAILDELIVERY.NOTALLOWED' => 'No se permite la opción de entrega en el correo regional.',
        'FREIGHT.GUARANTEE.NOTALLOWED' => 'No se permite la garantía de carga.',
        'SHIPMENT.CUSTOMSVALUE.REQUIRED' => 'Mercancía {COMMODITY_INDEX}: el valor de aduana es obligatorio. Actualice la información e inténtelo de nuevo.',
        'ORIGINZIPCODE.SERVICE.ERROR' => 'No se presta servicio a este código postal de origen.',
        'SERVICETYPE.NOT.ALLOWED' => 'El tipo de servicio seleccionado no se permite entre el origen y el destino. Actualice la información e inténtelo nuevamente.',
        'ACCOUNT.HAZARDOUSMATERIALS.NOTALLOWED' => 'El envío de material peligroso no está habilitado para su cuenta.',
        'DECLAREDVALUE.EXCEEDS.LIMIT' => 'El valor declarado supera el límite permitido. Actualice la información e inténtelo nuevamente.',
        'ENTERED.AMOUNT.INVALID' => 'El monto ingresado para el COD (Cobro contra entrega) del paquete {PACKAGE_INDEX} supera el límite máximo. Actualice la información e inténtelo nuevamente.',
        'PACKAGE.DECLAREDVALUE.EXCEEDED' => 'Paquete {PACKAGE_INDEX}: el valor declarado excede el límite de {CURRENCY_AMOUNT} {CURRENCY_TYPE} para el tipo de paquete.',
        'PACKAGETYPE.DECLAREDVALUE.EXCEEDED' => 'El valor declarado supera el límite de {CURRENCY_AMOUNT} {CURRENCY_TYPE} para el paquete {PACKAGE_INDEX}. Actualice la información e inténtelo nuevamente.',
        'SERVICE.TYPE.NOTAVAILABLE' => 'No se admite el tipo de servicio solicitado. Actualice la información e inténtelo de nuevo.',
        'SPECIALSERVICE.PICKUPMETHOD.NOT.ALLOWED' => '{SPECIAL_SERVICE_TYPE} no se permite con el método de recolección. Actualice la información e inténtelo nuevamente.',
        'SHIPMENT.DGHMDIVIOLATIONS.NOTALLOWED' => 'El servicio {SERVICE_TYPE} no está permitido debido a violaciones de mercancías peligrosas o hielo seco que limitan sus opciones de envío. Actualice la información e inténtelo nuevamente.',
        'CURRENCYAMOUNT.CURRENCYTYPE.EXCEEDSLIMIT' => 'El valor total para la aduana declarado ingresado supera el límite de {CURRENCY_AMOUNT} {CURRENCY_TYPE} para {SPECIAL_SERVICE_TYPE}. Actualice la información e inténtelo de nuevo.',
        'SERVICETYPE.VALUE.INVALID' => 'El tipo de producto es invalido.',
        'SERVICE.UNAVAILABLE.ERROR' => 'Servicio no disponible.',
        'RECIPIENT.COUNTRY.UNSERVICED' => 'No tenemos servicios disponibles para el pais de destino.',
        'REQUESTEDPACKAGELINEITEMS.WEIGHT.LIMITEXCEED' => 'El peso máximo permitido por Bulto es de 68 kg/150 lb.',
        'SYSTEM.UNAVAILABLE.EXCEPTION' => 'Servicio no disponible.',
        'EXPRESS.FREIGHT.NOTALLOWED' => 'Servicio no disponible.',
        'CARRIAGEVALUE.EXCEEDS.CUSTOMVALUE' => 'El valor de carga excede el maximo admitido.',
    
    ];

    private $db;



    public function __construct()
    {
        $this->db = new collection();
        $this->currentDate = new DateTime();
        $this->currentDate->setTimeZone(new DateTimeZone('America/Panama'));
        $this->currentDateString = $this->currentDate->format('Y-m-d H:i:s');
        $this->pickupDate = new DateTime();
        $this->setNextBussinessDay($this->pickupDate);
        $this->loadJWT();
        $this->setSenderData();
        //$this->generatePackageTotals(true);
    }

    private function loadJWT(): void
    {
        
        $key = $this->db->searchForce('fedex_auth_key_list', array('id' => 1));
        if ($key['cant_rows'] > 0) {
            $key = $key[0];
        }
        
        
        if (isset($key['key']) && $this->validateExpiration($key['date'], $key['expiration'])) {
            $this->jwt = $key['key'];
            return;
        }
        
        if ($data = $this->loadNewAuthorization()) {
            $response = json_decode($data, true);
            $this->db->update('fedex_auth_key_list', 1, array('key' => $response['access_token'], 'expiration' => $response['expires_in'], 'date' => date('Y-m-d H:i:s')));
            $this->jwt = $response['access_token'];
            return;
        }

        throw new Exception("Error Processing Auth Request", 5001);
    }
    private function loadNewAuthorization()
    {
        $content = 'grant_type=client_credentials&client_id=' . $this->apiKey . '&client_secret=' . $this->apiSecret;
        $ch = curl_init();
        curl_setopt_array($ch, [
            CURLOPT_URL => $this->url . '/oauth/token',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $content,
            CURLOPT_HTTPHEADER => [
                "Content-Type: application/x-www-form-urlencoded",
            ],
        ]);
        $data = curl_exec($ch);
        $error = curl_error($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if ($httpcode >= 200 && $httpcode < 300) {
            return $data;
        } else {
            $this->error = $error;
            return false;
        }
    }
    private function validateExpiration($date, $exp): bool
    {
        $result = false;
        $date = new DateTime($date);
        $difference = strtotime($this->currentDate->format('Y-m-d H:i:s')) - strtotime($date->setTimeZone(new DateTimeZone('America/Panama'))->format('Y-m-d H:i:s'));
        if ($difference < (intval($exp) - 360)) {
            $result = true;
        }
        return $result;
    }
    public function getAuthToken()
    {
        return $this->jwt;
    }
    private function setSenderData()
    {
        $this->senderData['address']['streetLines'][] = 'zona libre colon';
        $this->senderData['address']['streetLines'][] = 'bridgemarkets';
        $this->senderData['address']['city'] = 'colon';
        $this->senderData['address']['postalCode'] = '07097';
        $this->senderData['address']['countryCode'] = 'PA';
    }

    public function setCustomerData(Destination $destitanion)
    {
        $this->customerData['address']['streetLines'][] = $destitanion->address_1;
        $this->customerData['address']['streetLines'][] = $destitanion->address_2;
        $this->customerData['address']['city'] = $destitanion->city;
        $this->customerData['address']['postalCode'] = strval($destitanion->postcode);
        $this->customerData['address']['countryCode'] = $destitanion->get_countryCode();
        return $this;
    }

    private function setNextBussinessDay(DateTime &$date)
    {
        $date->modify($this->pickupDelay);
        switch ($date->format('D')) {
            case 'Sat':
                $date->modify('+2 day');
                break;
            case 'Sun':
                $date->modify('+1 day');
                break;
            default:
                break;
        }
    }

    private function createRequestBody()
    {

        $data['accountNumber']['value'] = $this->accountNumber;
        $data['rateRequestControlParameters']['returnTransitTimes'] = true;
        $data['requestedShipment'] = $this->getShipmentData();
        $data['carrierCodes'][] = 'FDXE';
        return json_encode($data);
    }
    private function getShipmentData()
    {
        $shipment['shipper'] = $this->senderData;
        $shipment['recipient'] = $this->customerData;
        $shipment['serviceType'] = $this->serviceType;
        $shipment['preferredCurrency'] = 'USD';
        $shipment['rateRequestType'][] = 'ACCOUNT';
        $shipment['shipDateStamp'] = $this->pickupDate->format('Y-m-d');
        $shipment['pickupType'] = $this->pickupType;
        $shipment['documentShipment'] = false;
        $shipment['packagingType'] = $this->packingType;
        $shipment['requestedPackageLineItems'] = $this->getPackages();
        $shipment['totalPackageCount'] = $this->package->getTotalItems();
        $shipment['totalWeight'] = $this->package->getTotalWeight();
        $shipment['customsClearanceDetail'] = $this->getCustomDetails();
        return $shipment;
    }

    private function getCustomDetails()
    {
        $details['commercialInvoice']['shipmentPurpose'] = 'COMMERCIAL';
        $details['freightOnValue'] = 'CARRIER_RISK';
        $details['dutiesPayment']['payor']['paymentType'] = 'SENDER';
        $details['dutiesPayment']['payor']['responsibleParty'] = $this->senderData;
        $details['dutiesPayment']['payor']['responsibleParty']['contact']['personName'] = 'Coordinador Logistica';
        $details['dutiesPayment']['payor']['responsibleParty']['contact']['emailAddress'] = 'logistica@bridgemarkets.com';
        $details['dutiesPayment']['payor']['responsibleParty']['contact']['phoneNumber'] = '63220194';
        $details['dutiesPayment']['payor']['responsibleParty']['contact']['companyName'] = 'Bridgemarkets';
        $details['dutiesPayment']['payor']['responsibleParty']['accountNumber']['value'] = $this->accountNumber;
        $details['commodities'] = $this->getItemsCommodities();
        return $details;
    }

    private function getItemsCommodities($extendedData = false)
    {
        $data = [];
        if ($extendedData) {
            /**
             * extended information
             */
        } else {
            foreach ($this->package->getItems() as $item) {
                if ($item instanceof PackageItem) {
                    $itemData['description'] = $item->get_product_name();
                    $itemData['customsValue']['amount'] = $item->get_price() * $item->get_quantity();
                    $itemData['customsValue']['currency'] = 'USD';
                    $itemData['weight']['units'] = 'KG';
                    $itemData['weight']['value'] = $item->get_weight();
                    $itemData['numberOfPieces'] = $item->get_moq() * $item->get_quantity();
                    $itemData['quantity'] = $item->get_quantity();
                    $itemData['countryOfManufacture'] = $item->get_product_origin_country();
                    $itemData['quantityUnits'] = $item->get_quantity();
                    $itemData['name'] = $item->get_product_name();
                    array_push($data, $itemData);
                    unset($itemData);
                }
            }
        }
        return $data;
    }
    private function getPackages($extendedData = false)
    {
        $data = [];
        if ($extendedData) {
            /**
             * extended information
             */
        } else {
            foreach ($this->package->getItems() as $item) {
                if ($item instanceof PackageItem) {
                    $itemData['subPackagingType'] = 'BOX';
                    $itemData['groupPackageCount'] = $item->get_quantity();
                    $itemData['declaredValue']['amount'] = $item->get_price();
                    $itemData['declaredValue']['currency'] = 'USD';
                    $itemData['weight']['units'] = 'KG';
                    $itemData['weight']['value'] = $item->get_weight();
                    $itemData['dimensions']['length'] = $item->get_length();
                    $itemData['dimensions']['width'] = $item->get_width();
                    $itemData['dimensions']['height'] = $item->get_height();
                    $itemData['dimensions']['units'] = 'CM';
                    array_push($data, $itemData);
                    unset($itemData);
                }
            }
        }
        return $data;
    }
    public function export_shipping()
    {
    }

    public function import_shipping()
    {
    }
    public function errorResponse()
    {

        $res['error'] = true;
        $res['method_title'] = 'FedEx Connect Plus';
        $res['include_taxes'] = 'No incluye impuestos en destino';
        $res['description'] = 'FedEx International Connect Plus';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/Fedex.png';
        $res['message'] = $this->errorcodes[$this->error];
        $prod['type'] = 'air';
        $prod['productName'] = (isset($this->errorcodes[$this->error]))?$this->errorcodes[$this->error]:'Este servicio no esta disponible en este momento';
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = 0;
        $prod['volumetric'] = $this->package->getTotalCBM();
        $prod['deliveryCapabilities']['totalTransitDays'] = 0;
        $res['options'][] = $prod;
        return $res;

    }

    public function generateResponseSchema(array $data, bool $include_taxes = false)
    {
        $res = [];
        if (!$data || empty($data) || empty($data['output']['rateReplyDetails'])) {
            return $this->errorResponse();
        }
        $res['method_title'] = 'FedEx Connect Plus';
        $res['description'] = 'FedEx International Connect Plus';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/proveedores_logisticos/Fedex.png';
        $res['message'] = 'Los cargos arancelarios le serán comunicados en destino.';
        foreach ($data['output']['rateReplyDetails'] as $product) {
            $prod['type'] = 'air';
            $prod['productName'] = $product['serviceDescription']['names'][0]['value'] . ' - 7-10 días hábiles. Este plazo de entrega está sujeto a verificación de inventario por parte del vendedor y al plazo de entrega del proveedor logístico.';
            $prod['totalPrice']['priceCurrency'] = 'USD';
            $prod['totalPrice']['price'] = round($product['ratedShipmentDetails'][0]['totalNetChargeWithDutiesAndTaxes'], 2);
            $prod['volumetric'] = $this->package->getTotalCBM();
            $prod['deliveryCapabilities']['totalTransitDays'] = $this->daysFromNow($product['commit']['dateDetail']['dayFormat']) ?? 0;
            //$prod['details'][] = $this->insertDetail('Impuesto de Nacionalización', 'Se calcularán en destino', 0);
            $prod['details'][] = $this->insertDetail('Envío', '', round($product['ratedShipmentDetails'][0]['totalNetChargeWithDutiesAndTaxes'], 2));
            $res['options'][] = $prod;
            unset($prod);
        }
        return $res;
    }

    private function daysFromNow($date): int
    {
        $estimateDateDelivery = new DateTime($date);
        $dateDiff = $this->currentDate->diff($estimateDateDelivery);
        return $dateDiff->days;
    }

    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }

    public function checkout_rate(Destination $destitanion, Package $package, bool $include_taxes)
    {
        $this->package = $package;
        $this->setCustomerData($destitanion);
        $responseCurl = new CurlRequest();
        //die(var_dump($this->createRequestBody()));
        $responseCurl->addHeaders('Authorization: Bearer ' . $this->jwt);
        
        $responseCurl->request($this->url . '/rate/v1/rates/quotes', 'POST', $this->createRequestBody());
        if (!$responseCurl->status()) {
            $this->status = false;
            $errors = $responseCurl->toObject()['errors'][0];
            $this->error = $errors['code'];

            return $this->errorResponse();
        }
        return $this->generateResponseSchema($responseCurl->toObject(), $include_taxes);
    }
}
