<?php 

class MultiBox_aereo_class implements Currier
{
    private $db;
    private $package;

    private $subTotal = 0;
    private $total = 0;
    private $taxes = 0.1;
    private $logisticCost= 0;
    private $bankFee = 0.05;
    private $margin = 0.1;
    private $handling = 9.5;
    private $minimum_price = 35.5;
    private $pricePerWeightRange= [
        
        600=>10.5,
        1500 => 10,
        3000 => 9.5,
        'other'=>9
    ];
    function __construct()
    {
        $this->db = new collection;
    }
    
    public function generateResponseSchema(array $data, bool $include_taxes = false)
    {
        $res = [];

        $res['method_title'] = 'Multibox';
        $res['description'] = 'El tiempo estimado de entrega para envíos aéreos es de 7 a 14 días hábiles, considerando el proceso de consolidación, desconsolidación y trámites aduanales.';
        $res['include_taxes'] = ($include_taxes)?'Incluye impuestos en destino':'No incluye impuestos en destino';
        $res['message'] = 'Tiempos por destino:
Caracas y La Guaira: Entrega estimada en 7 días hábiles después de la fecha de vuelo.
Otras ciudades: Entrega estimada entre 9 y 14 días hábiles después de la fecha de vuelo.
Nota: Estos tiempos son aproximados y pueden verse afectados por factores logísticos y aduanales.';
        $res['provider_image'] = 'https://bmbucket01.s3.ca-central-1.amazonaws.com/uploads/2024/10/18aa4da0-b4ce-4ed2-aeb7-faab96d07b21.png';
        $prod['type'] = 'air';
        $prod['productName'] = 'El tiempo estimado de entrega para envíos aéreos es de 7 a 14 días hábiles, considerando el proceso de consolidación, desconsolidación y trámites aduanales.';
        $prod['totalPrice']['priceCurrency'] = 'USD';
        $prod['totalPrice']['price'] = round($this->total, 2);
        $prod['volumetric'] = $this->package->getTotalCBM()['air'];
        $prod['deliveryCapabilities']['totalTransitDays'] = 0;
        $prod['details'][] = $this->insertDetail('Trámite logístico', 'Trámite logístico', $this->logisticCost + $this->handling);
        //$prod['details'][] = $this->insertDetail('Impuesto de Nacionalización', 'Cargos por nacionalización', 0);
        $prod['details'][] = $this->insertDetail('Envío', '', $this->subTotal);
        $res['options'][] = $prod;
        return $res;
    }
    public function errorResponse(){
        return;
    }

    public function checkout_rate(Destination $destitanion, Package $package, bool $include_taxes){
        $this->package = $package;
        $this->calculateSubTotal($package);
        $this->recalculate($package);

        if ($this->total != 0) {
            return $this->generateResponseSchema([], $include_taxes);
        }
    }

    private function recalculate(Package $package){

        $total_taxes=$this->calculateTaxes($package);
        $this->total = $this->subTotal + $total_taxes + $this->handling;
        $this->total= (1+$this->bankFee)*$this->total; 

    }
    private function calculateSubTotal(Package $package){
     $volWeight= $package->getTotalCBM()['air'];
     $realWeight = $package->getTotalWeight();
     $weightToCharge = max($volWeight,$realWeight);
     $rate = $this->getRatePerWeight($weightToCharge);
     $this->subTotal = $rate*$weightToCharge;
    }
    private function calculateTaxes(Package $package){
        $marginCost = $this->subTotal*$this->margin;
        $itbmsCost = $package->getTotalOrder()*$this->taxes;
        $this->logisticCost =  $marginCost + $itbmsCost;
        return $this->logisticCost;
    }
    private function getRatePerWeight($weight){
        if(!isset($weight)){
            return  0;
        }
        if($weight <= 3){
            return $this->minimum_price;
        }
        foreach($this->pricePerWeightRange as $key => $value){
            if($weight <= $key){
                return $value;
            }
        }
        return $this->pricePerWeightRange['other'];
    }

    public function insertDetail($name = '', $text = '', $value = '')
    {
        $dt['name'] = $name;
        $dt['msg'] = $text;
        $dt['value'] = round($value, 2);
        return $dt;
    }
}