<?php

class Store
{
    private $db;
    private $user_id = 0;
    private $list = [];
    private $following = [];
    private $id_list = [];
    private $full_list = [];
    private $queneau_list = [];
    private $categories = [];
    private $meta_fields = [
        'dokan_enable_selling' => 'can_sell',
        'dokan_profile_settings' => 'dokan_profile_settings',
        'dokan_store_name' => 'dokan_store_name',
        'dokan_geo_latitude' => 'dokan_geo_latitude',
        'dokan_geo_longitude' => 'dokan_geo_longitude',
        'dokan_geo_public' => 'dokan_geo_public',
        'dokan_geo_address' => 'dokan_geo_address',

    ];
    private $minimum_meta_fields = [
        'dokan_enable_selling' => 'can_sell',
        'dokan_profile_settings' => 'dokan_profile_settings',
        'dokan_store_name' => 'dokan_store_name',
        'dokan_geo_latitude' => 'dokan_geo_latitude',
        'dokan_geo_longitude' => 'dokan_geo_longitude',
        'dokan_geo_public' => 'dokan_geo_public',
        'dokan_geo_address' => 'dokan_geo_address',
    ];

    private $dokan_meta_avalible = [
        'store_name',
        'social',
        'phone',
        'show_email',
        'address',
        'location',
        'banner',
        'gravatar',
        'store_seo',
        'dokan_store_time_enabled',
        'dokan_store_open_notice',
        'dokan_store_close_notice',
        'dokan_store_time',
        //'categories',
        'vendor_biography',
        'show_support_btn_product',
        'dokan_store_name',
        'support_btn_name',
        'show_support_btn',
        'whatsapp_number',
    ];

    public function __construct()
    {
        $this->db = new collection();
        try {
            $auth = new auth();
            if ($auth->verify($auth->found())->status()) {
                $this->user_id = $auth->getUser();
            }
        } catch (\Throwable $th) {
            $this->user_id = 0;
        }
    }

    public function followUp($store)
    {
        if ($this->user_id == 0) {
            throw new Exception("Usuario no  puede ser Nulo", 250);
            return;
        }
        if (!$store || $store == '') {
            throw new Exception("Tienda no  puede ser Nulo", 251);
            return;
        }
        $res = $this->db->search('bm_follow_store', array('user_id' => $this->user_id, 'vendor_id' => $store));
        if ($res['cant_rows'] > 0) {
            throw new Exception("Usted ya sigue esta tienda", 251);
            return;
        }
        $data = array('user_id' => $this->user_id, 'vendor_id' => $store);
        $this->db->insert('bm_follow_store', $data);
        return $this->getFollowUpStores();
    }
    public function followDown($store)
    {
        if ($this->user_id == 0) {
            throw new Exception("Usuario no  puede ser Nulo", 250);
            return;
        }
        if (!$store || $store == '') {
            throw new Exception("Tienda no  puede ser Nulo", 251);
            return;
        }
        $res = $this->db->search('bm_follow_store', array('user_id' => $this->user_id, 'vendor_id' => $store));
        if ($res['cant_rows'] <= 0) {
            throw new Exception("Usted no sigue esta tienda", 251);
            return;
        }
        $data = array('user_id' => $this->user_id, 'vendor_id' => $store);
        $this->db->delete('bm_follow_store', $res[0]['id']);
        return $this->getFollowUpStores();
    }

    public function getFollowUpStores()
    {
        $this->search_routine_following();
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->compact();
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }

    private function getFollowing()
    {
        if ($this->user_id != 0) {
            $query = str_replace(
                'ovmdw_',
                TABLE_PREFIX,
                "SELECT
                bm_follow_store.vendor_id
                FROM
                bm_follow_store
                WHERE
                bm_follow_store.user_id = '" . $this->user_id . "'
                GROUP BY
                bm_follow_store.vendor_id           
                "
            );
            $data = $this->db->plainQuery($query, false, true);
            for ($i = 0; $i < $data['cant_rows']; $i++) {
                array_push($this->following, $data[$i]['vendor_id']);
            }
        }


        foreach ($this->full_list as $key => $value) {
            if (in_array($key, $this->following)) {
                $this->full_list[$key]['following'] = true;
            } else {
                $this->full_list[$key]['following'] = false;
            }
        }
    }

    public function autocomplete($keyword = '', $count = 5)
    {
        $this->search_routine_autocomplete($keyword, $count);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->compact();
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }

    public function getStoreCategories()
    {
        $this->getCategoriesMaster();
        return array('categories' => $this->categories, 'count' => count($this->categories));
    }

    public function getAll($page = 1, $count = 12, $order = false)
    {
        $this->search_routine($page, $count);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact($page, $count,$order);
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }

    public function searchByCategory($keyword = '', $page = 1, $count = 12, $order = false)
    {
        $this->search_routine_by_category($keyword, $page, $count);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact($page, $count,$order);
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }

    public function searchByName($keyword = '', $page = 1, $count = 12, $order = false)
    {
        $this->search_routine_by_vendor($keyword, $page, $count);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact($page, $count,$order);
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }

    public function detailStore($keyword = '')
    {
        $this->search_routine_by_vendor_slug($keyword);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact();
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }
    public function getStoreById($id = '')
    {
        $this->searchRoutineById($id);
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact();
        return $this->list;
    }

    public function getHighLights()
    {
        $this->highlights();
        $this->getMeta();
        $this->getCategories();
        $this->getPrimaryImages();
        $this->getFollowing();
        $this->_productsCount();
        $this->compact();
        return array('stores' => $this->list, 'count' => count($this->id_list));
    }



    private function storeTaxonomiesList($data, &$arr_cats)
    {
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $store = $data[$i]['object_id'];
            if ($this->full_list[$store]) {
                array_push($this->full_list[$store]['categories'], array(
                    'id' => $data[$i]['term_id'],
                    'name' => $data[$i]['name'],
                    'slug' => $data[$i]['slug'],
                ));
                array_push($arr_cats, $data[$i]['slug']);
            }
        }
    }

    private function getCategories($save = false)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_terms.term_id,
            ovmdw_terms.`name`,
            ovmdw_terms.slug,
            ovmdw_term_taxonomy.taxonomy,
            ovmdw_term_relationships.object_id
            FROM
            ovmdw_term_relationships
            INNER JOIN ovmdw_term_taxonomy ON ovmdw_term_taxonomy.term_taxonomy_id = ovmdw_term_relationships.term_taxonomy_id
            INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
            WHERE
            ovmdw_term_relationships.object_id IN (" . implode(",", $this->id_list) . ") AND
            ovmdw_term_taxonomy.taxonomy = 'store_category'
            ORDER BY
            OvMdW_term_taxonomy.count DESC           
            "
        );
        $data = $this->db->plainQuery($query);
        $arr_cats = [];
        $this->storeTaxonomiesList($data, $arr_cats);
        unset($data);
        unset($arr_cats);
        unset($query);
    }

    private function fillPrimaryList($data, $i)
    {
        $key = $data[$i]['ID'];
        $this->id_list[$key]['vendor_id'] = $data[$i]['ID'];
        $this->id_list[$key]['name'] = $data[$i]['display_name'];
        $this->id_list[$key]['slug'] = $data[$i]['user_nicename'];
        $this->id_list[$key]['email'] = $data[$i]['user_email'];
        $this->id_list[$key]['status'] = $data[$i]['user_status'];
        $this->id_list[$key]['created_at'] = $data[$i]['user_registered'];
        $this->id_list[$key]['modified_at'] = $data[$i]['post_modified'];
        $this->id_list[$key]['categories'] = [];
        $this->id_list[$key]['products'] = '0';
    }

    private function getMeta($quick = false)
    {
        $keys = ($quick) ? $this->minimum_meta_fields : $this->meta_fields;
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            OvMdW_usermeta.user_id,
            OvMdW_usermeta.meta_key,
            OvMdW_usermeta.meta_value
            FROM
            OvMdW_usermeta
            WHERE
            OvMdW_usermeta.user_id IN (" . implode(',', array_keys($this->full_list)) . ")
            AND
            OvMdW_usermeta.meta_key IN ('" . implode("','", array_keys($keys)) . "')
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->manageMeta($data[$i]);
        }
        unset($data);
        unset($query);
    }

    private function manageMeta($meta_data)
    {
        $id = $meta_data['user_id'];
        switch ($meta_data['meta_key']) {
            case 'dokan_profile_settings':
                $fields = $this->handle_values($meta_data['meta_value']);
                //die(json_encode($fields));
                foreach ($fields as $key => $value) {
                    if (in_array($key, $this->dokan_meta_avalible)) {
                        switch ($key) {
                            case 'banner':
                                $this->queneau_list[$value][$id] = 'banner';
                                break;
                            case 'gravatar':
                                $this->queneau_list[$value][$id] = 'gravatar';
                                break;

                            default:
                                $this->full_list[$id][$key] = $value;
                                break;
                        }
                    }
                }
                break;
            default:
                $field = $this->meta_fields[$meta_data['meta_key']];
                $this->full_list[$id][$field] = $this->handle_values($meta_data['meta_value']);
                break;
        }
    }
    private function handle_values($value)
    {
        $m = new MetaValues();
        $dt =  $m->maybe_unserialize($value);
        unset($m);
        return $dt;
    }
    private function getPrimaryImages()
    {
        if (count(array_unique(array_filter(array_keys($this->queneau_list)))) == 0) {
            return false;
        }
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.post_id,
            ovmdw_postmeta.meta_value
            FROM
            ovmdw_postmeta
            WHERE
            ovmdw_postmeta.post_id IN (" . implode(",", array_unique(array_filter(array_keys($this->queneau_list)))) . ") AND
            ovmdw_postmeta.meta_key = '_wp_attached_file'
            GROUP BY
            ovmdw_postmeta.meta_id            
            "
        );
        $data = $this->db->plainQuery($query);

        foreach ($this->queneau_list as $attach_id => $product) {
            foreach ($product as $user_id => $type) {
                for ($i = 0; $i < $data['cant_rows']; $i++) {
                    if ($attach_id == $data[$i]['post_id']) {
                        //echo 'exist---/---';
                        //$url = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . str_replace('-scaled', '', $data[$i]['meta_value']);
                        $url = 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . str_replace('-scaled', '', $data[$i]['meta_value']);
                        $this->full_list[$user_id][$type] = $url;
                        $i = $data['cant_rows'];
                    }
                }
            }
        }
        unset($data);
        unset($query);

        //die();
    }

    private function compact($page = 1, $count = 12, $order=false)
    {
        $parent = [];

        foreach ($this->full_list as $key => $value) {
            $parent[$key][] = $value;
        }
        $this->full_list = [];
        if($order){
            usort($parent, function ($a, $b) {
                if (intval($a[0]['products']) == intval($b[0]['products'])) {
                    return 0;
                }
                return (intval($a[0]['products']) > intval($b[0]['products'])) ? -1 : 1;
            });
        }        
        $parent = array_slice($parent, ($page - 1) * $count, $count, true);
        foreach ($parent as $key => $value) {
            if ($value[0]['name'] != '') {
                array_push($this->list, $value[0]);
            }
        }
        
        
        
        $parent = [];
    }

    private function highlights()
    {

        $stores = [5376, 640, 742, 260, 4, 91, 67, 754, 52, 363, 109, 892];
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'dokan_enable_selling' AND
            OvMdW_usermeta.meta_value = 'yes' AND
            OvMdW_users.user_status = 0 AND
            OvMdW_users.ID IN(" . implode(',', $stores) . ")
            ORDER BY
            OvMdW_users.display_name ASC
            "
        );
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }


    private function search_routine($page = 1, $count = 12)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'dokan_enable_selling' AND
            OvMdW_usermeta.meta_value = 'yes' AND
            OvMdW_users.user_status = 0
            ORDER BY
            OvMdW_users.display_name ASC
            "
        );
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        //$this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function search_routine_following()
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
                OvMdW_users.ID,
                OvMdW_users.user_login,
                OvMdW_users.user_nicename,
                OvMdW_users.user_email,
                OvMdW_users.user_url,
                OvMdW_users.user_registered,
                OvMdW_users.user_activation_key,
                OvMdW_users.display_name,
                OvMdW_users.user_status
                    FROM
                    OvMdW_users
                    INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
                    INNER JOIN bm_follow_store ON bm_follow_store.vendor_id = OvMdW_users.ID
                    WHERE
                    OvMdW_usermeta.meta_key = 'dokan_enable_selling' AND
                    OvMdW_usermeta.meta_value = 'yes' AND
                    OvMdW_users.user_status = 0 AND
                    bm_follow_store.user_id = '" . $this->user_id . "'
                    ORDER BY
                OvMdW_users.display_name ASC
            "
        );
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = $this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function search_routine_autocomplete($vendor_name = '', $count = 12)
    {
        if ($vendor_name == '' || $vendor_name == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_name = str_replace(' ', '%', trim($vendor_name));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'dokan_store_name' AND
            OvMdW_usermeta.meta_value LIKE '%" . $this->db->sanitizeData($vendor_name) . "%' AND
            OvMdW_users.ID IN (SELECT e.user_id FROM OvMdW_usermeta as e WHERE e.meta_key = 'dokan_enable_selling' AND
            e.meta_value = 'yes') AND
            OvMdW_users.user_status = 0
            ORDER BY
            OvMdW_usermeta.meta_value ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, 0, $count, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    private function searchRoutineById($id = '')
    {
        if ($id == '' || $id == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_name = str_replace(' ', '%', trim($id));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            WHERE
            OvMdW_users.ID = '" . $this->db->sanitizeData($vendor_name) . "' AND
            OvMdW_users.user_status = 0
            LIMIT 1
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, 0, 1, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_routine_by_vendor_slug($vendor_name = '')
    {
        if ($vendor_name == '' || $vendor_name == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_name = str_replace(' ', '%', trim($vendor_name));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            WHERE
            OvMdW_users.user_nicename = '" . $this->db->sanitizeData($vendor_name) . "' AND
            OvMdW_users.user_status = 0
            LIMIT 1
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        $this->full_list = array_slice($this->id_list, 0, 1, true);
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_routine_by_vendor($vendor_name = '', $page = 1, $count = 12)
    {
        if ($vendor_name == '' || $vendor_name == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $vendor_name = str_replace(' ', '%', trim($vendor_name));

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_usermeta.meta_key = 'dokan_store_name' AND
            OvMdW_usermeta.meta_value LIKE '%" . $this->db->sanitizeData($vendor_name) . "%' AND
            OvMdW_users.user_status = 0
            ORDER BY
            OvMdW_usermeta.meta_value ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        //$this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->full_list =$this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }
    private function search_routine_by_category($slug = '', $page = 1, $count = 12)
    {
        if ($slug == '' || $slug == ' ') {
            $this->list = [];
            $this->id_list = [];
            $this->full_list = [];
            return $this;
        }

        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "
            SELECT
            OvMdW_users.ID,
            OvMdW_users.user_login,
            OvMdW_users.user_nicename,
            OvMdW_users.user_email,
            OvMdW_users.user_url,
            OvMdW_users.user_registered,
            OvMdW_users.user_activation_key,
            OvMdW_users.display_name,
            OvMdW_users.user_status
            FROM
            OvMdW_users
            INNER JOIN OvMdW_usermeta ON OvMdW_usermeta.user_id = OvMdW_users.ID
            WHERE
            OvMdW_users.ID IN (
                SELECT
                    OvMdW_term_relationships.object_id
                    FROM
                    OvMdW_terms
                    INNER JOIN OvMdW_term_taxonomy ON OvMdW_term_taxonomy.term_id = OvMdW_terms.term_id
                    INNER JOIN OvMdW_term_relationships ON OvMdW_term_relationships.term_taxonomy_id = OvMdW_term_taxonomy.term_taxonomy_id
                    INNER JOIN OvMdW_users ON OvMdW_users.ID = OvMdW_term_relationships.object_id
                    WHERE
                    OvMdW_terms.slug = '" . $this->db->sanitizeData($slug) . "'
                    GROUP BY
                    OvMdW_term_relationships.object_id
            )
        AND OvMdW_usermeta.meta_key = 'dokan_enable_selling'
        AND OvMdW_usermeta.meta_value = 'yes'
        AND OvMdW_users.user_status = 0
            ORDER BY
            OvMdW_users.display_name ASC
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $this->fillPrimaryList($data, $i);
        }
        //$this->full_list = array_slice($this->id_list, ($page - 1) * $count, $count, true);
        $this->full_list =$this->id_list;
        $this->id_list = array_keys($this->id_list);
        unset($data);
        unset($query);
        return $this;
    }

    public function getImage($id)
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            ovmdw_postmeta.meta_value AS img
            FROM
            ovmdw_posts
            INNER JOIN ovmdw_postmeta ON ovmdw_postmeta.post_id = ovmdw_posts.ID
            WHERE
            ovmdw_posts.ID IN ((SELECT
            ovmdw_termmeta.meta_value
            FROM
            ovmdw_termmeta
            WHERE
            ovmdw_termmeta.meta_key = 'thumbnail_id' AND
            ovmdw_termmeta.term_id = " . $id . ")) AND
            ovmdw_postmeta.meta_key = '_wp_attached_file'
            GROUP BY
            ovmdw_postmeta.meta_id,
            ovmdw_postmeta.meta_key,
            ovmdw_postmeta.meta_value
            LIMIT 1"
        );
        $data = $this->db->plainQuery($query);
        if ($data['cant_rows']) {
            //return 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . $data[0]['img'];
            return 'https://d1u9bbhcasux7c.cloudfront.net/uploads/' . $data[0]['img'];
        } else {
            return '';
        }
    }

    private function getCategoriesMaster()
    {
        $parents = [];
        $children = [];
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
        ovmdw_term_taxonomy.term_id,
        ovmdw_terms.slug,
        ovmdw_terms.`name` as description,
        ovmdw_term_taxonomy.parent,
        ovmdw_term_taxonomy.count
        FROM
        ovmdw_term_taxonomy
        INNER JOIN ovmdw_terms ON ovmdw_terms.term_id = ovmdw_term_taxonomy.term_id
        WHERE
        ovmdw_term_taxonomy.taxonomy = 'store_category' AND
        ovmdw_term_taxonomy.parent = 0
        GROUP BY
        ovmdw_term_taxonomy.term_id
        ORDER BY
        ovmdw_terms.`name` ASC"
        );
        $data = $this->db->plainQuery($query);

        for ($i = 0; $i < $data['cant_rows']; $i++) {
            array_push($parents, $data[$i]['term_id']);
        }


        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $data[$i]['image'] = $this->getImage($data[$i]['term_id']);
            $data[$i]['children'] = [];
            array_push($this->categories, $data[$i]);
        }
        return $this;
    }

    private function _productsCount()
    {
        $query = str_replace(
            'ovmdw_',
            TABLE_PREFIX,
            "SELECT
            COUNT(OvMdW_posts.ID) AS Products,
            OvMdW_posts.post_author,
            OvMdW_posts.post_status,
            OvMdW_posts.post_type
        FROM
            OvMdW_posts
        WHERE
                OvMdW_posts.post_type = 'product' AND OvMdW_posts.post_status = 'publish' AND OvMdW_posts.post_author IN(" . implode(',', $this->id_list) . "    ) AND OvMdW_posts.ID NOT IN(
        SELECT
            OvMdW_postmeta.post_id
        FROM
            OvMdW_postmeta
        WHERE
            OvMdW_postmeta.meta_key = '_visibility' AND OvMdW_postmeta.meta_value = 'hidden'
        GROUP BY
            OvMdW_postmeta.post_id
            )
        GROUP BY
        OvMdW_posts.post_author
        HAVING
            OvMdW_posts.post_status = 'publish' AND OvMdW_posts.post_type = 'product'
            "
        );
        //die($query);
        $data = $this->db->plainQuery($query);
        for ($i = 0; $i < $data['cant_rows']; $i++) {
            $store = $data[$i]['post_author'];
            if ($this->full_list[$store]) {
                $this->full_list[$store]['products'] = $data[$i]['Products'];
            }
        }
    }
}
