<?php 
class RestResponse {
    private $headers = [];
    private $response = [];
    private $body = '';
    private $error = '';
    private $errorMsg = [];
    private $message = 'OK';
    private $token = '';
    private $status = false;
    private $response_type='json';

    public function __construct($status=false, $body='') {
        $this->status = $status;
        $this->body=$body;
    }

    public function toJson()
    {
        $this->response_type='json';
        $this->setHeader('Content-Type: application/json');
        return $this;
    }
    public function toString()
    {
        $this->setHeader('Content-Type: text/html');
        $this->response_type='string';
        return $this;
    }
    public function encrypt()
    {
        $this->setHeader('Content-Type: application/json');
        $this->response_type='encrypt';
        return $this;
    }
    public function debug()
    {
        $this->response_type='debug';
        return $this;
    }
    public function setHeader($value)
    {
        $this->headers[]=$value;
        return $this;
    }

    public function setMessage($value)
    {
        $this->message=$value;
        return $this;
    }

    public function setError($value)
    {
        $this->errorMsg[]=$value;
        return $this;
    }
    public function setToken($value)
    {
        $this->token=$value;
        return $this;
    }
    public function setBody($value)
    {
        $this->body=$value;
        return $this;
    }
    public function setStatus($value)
    {
        $this->body=$value;
        return $this;
    }

    private function printResponse()
    {
        $this->response['body']=$this->body;
        $this->response['token']=$this->token;
        $this->response['status']=$this->status;
        $this->response['message']=$this->message;
        $this->response['error']=$this->error;
        $this->response['error_details']=$this->errorMsg;
        switch ($this->response_type) {
            case 'json':
                echo json_encode($this->response,JSON_BIGINT_AS_STRING);
                break;
            case 'string':
                echo $this->response['body'];
                break;
            case 'debug':
                echo print_r($this->response);
                break;
            case 'encrypt':
                $this->response['body']=secureToke::encrypt($this->body);
                echo json_encode($this->response,JSON_BIGINT_AS_STRING);
                break;
            default:
            echo json_encode($this->response);
                break;
        }
    }


    public function execute()
    {
        if (isset($_SERVER['HTTP_ORIGIN'])) {
            header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
            header('Access-Control-Allow-Credentials: true');
            header('Access-Control-Max-Age: 86400');
            header('Access-Control-Allow-Methods: GET, POST, PUT, DELETE, OPTIONS');
            header('Access-Control-Allow-Headers: Origin, Content-Type, X-Auth-Token');
        }
        if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
            if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD']))
                header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
            if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']))
                header("Access-Control-Allow-Headers: {$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']}");
            exit(0);
        }
        if(!$this->status){
            $this->error= $this->body;
            $this->body=null;
            foreach ($this->headers as $key => $value) {
                header($value);
            }
            $this->printResponse();
            return;
        }
        $this->setHeader('HTTP/1.1 200 OK');
        foreach ($this->headers as $key => $value) {
            header($value);
        }        
        $this->printResponse();
        return;
    }
}